# -*- coding: utf-8 -*-
"""Nonwords.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      nonwords
date:       2014-06-10 12:31:19
categories: writing
---

Nonwords.

"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "nonwords.misc"
    msg = "Nonword, try '{}'."

    preferences = [

        ["doubtless' or 'undoubtedly",   ["doubtlessly"]],
        ["analysis",                     ["analyzation"]],
        ["annoyance",                    ["annoyment"]],
        ["confirmand",                   ["confirmant"]],
        ["confirmands",                  ["confirmants"]],
        ["converse",                     ["conversate"]],
        ["cranded",                      ["crained"]],
        ["disbursement' or 'dispersal",  ["dispersement"]],
        ["discomfort' or 'discomfiture", ["discomforture"]],
        ["effrontery",                   ["affrontery"]],
        ["forbearance",                  ["forebearance"]],
        ["improper",                     ["improprietous"]],
        ["inclement",                    ["inclimate"]],
        ["relatively low price' or 'affordability", ["relative inexpense"]],
        ["inimical",                     ["inimicable"]],
        ["regardless",                   ["irregardless"]],
        ["minimize",                     ["minimalize"]],
        ["minimized",                    ["minimalized"]],
        ["minimizes",                    ["minimalizes"]],
        ["minimizing",                   ["minimalizing"]],
        # muchly
        ["optimize",                     ["optimalize"]],
        ["paralysis",                    ["paralyzation"]],
        ["pettifog",                     ["pettifogger"]],
        ["proper",                       ["proprietous"]],
        ["quell' or 'quash",             ["squelch"]],
        ["seldom",                       ["seldomly"]],
        # slinged
        ["thus",                         ["thusly"]],
        ["categorically",                ["uncategorically"]],
        ["undoubtedly' or 'indubitably", ["undoubtably"]],
        ["unequivocal",                  ["unequivocable"]],
        ["mercilessly",                  ["unmercilessly"]],
        ["unrelentingly' or relentlessly", ["unrelentlessly"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



# -*- coding: utf-8 -*-
"""Bureaucratese.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      bureaucratese
date:       2014-06-10 12:31:19
categories: writing
---

Bureaucratese.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "misc.bureaucratese"
    msg = u"'{}' is bureaucratese."

    bureaucratese = [
        "meet with your approval",
        "meets with your approval",
    ]

    return existence_check(text, bureaucratese, err, msg, join=True)



# -*- coding: utf-8 -*-
"""Password in plain text.

---
layout:     post
source:     ???
source_url: ???
title:      password in plain text
date:       2014-06-10 12:31:19
categories: writing
---

Don't put pass
"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "security.password"
    msg = u"Don't put passwords in plain text."

    pwd_regex = "[:]? [\S]{6,30}"

    password = [
        "the password is{}".format(pwd_regex),
        "my password is{}".format(pwd_regex),
        "the password's{}".format(pwd_regex),
        "my password's{}".format(pwd_regex),
        "^[pP]assword{}".format(pwd_regex),
    ]

    return existence_check(text, password, err, msg)



# -*- coding: utf-8 -*-

u"""Cliches are cliché."""

from proselint.tools import memoize, existence_check


@memoize
def check_cliches_garner(text):
    """Check the text.

    source:     Garner's Modern American Usage
    source_url: http://bit.ly/1T4alrY
    """
    err = "cliches.garner"
    msg = u"'{}' is cliché."

    cliches = [
        "a fate worse than death",
        "alas and alack",
        "at the end of the day",
        "bald-faced lie",
        "between a rock and a hard place",
        "between Scylla and Charybdis",
        "between the devil and the deep blue see",
        "betwixt and between",
        "blissful ignorance",
        "blow a fuse",
        "bulk large",
        "but that's another story",
        "cast aspersions",
        "chase a red herring",
        "comparing apples and oranges",
        "compleat",
        "conspicuous by its absence",
        "crystal clear",
        "cutting edge",
        "decision-making process",
        "dubious distinction",
        "duly authorized",
        "eyes peeled",
        "far be it from me",
        "fast and loose",
        "fills the bill",
        "first and foremost",
        "for free",
        "get with the program",
        "gilding the lily",
        "have a short fuse",
        "he's got his hands full",
        "his own worst enemy",
        "his work cut out for him",
        "hither and yon",
        "Hobson's choice",
        "horns of a dilemma",
        "if you catch my drift",
        "in light of",
        "in the final analysis",
        "in the last analysis",
        "innocent bystander",
        "it's not what you know, it's who you know",
        "last but not least",
        "make a mockery of",
        "male chauvinism",
        "moment of truth",
        "more in sorrow than in anger",
        "more sinned against than sinning",
        "my better half",
        "nip in the bud",
        "olden days",
        "on the same page",
        "presidential timber",
        "pulled no punches",
        "quantum jump",
        "quantum leap",
        "redound to one's credit",
        "redound to the benefit of",
        "sea change",
        "shirked his duties",
        "six of one, half a dozen of the other",
        "stretched to the breaking point",
        "than you can shake a stick at",
        "the cream of the crop",
        "the cream rises to the top",
        "the straw that broke the camel's back",
        "thick as thieves",
        "thinking outside the box",
        "thought leaders?",
        "throw the baby out with the bathwater",
        "various and sundry",
        "viable alternative",
        "wax eloquent",
        "wax poetic",
        "we've got a situation here",
        "whet (?:the|your) appetite",
        "wool pulled over our eyes",
        "writ large",
    ]
    return existence_check(text, cliches, err, msg, join=True)


@memoize
def check_cliches_write_good(text):
    """Check the text.

    source:     write-good
    source_url: https://github.com/btford/write-good
    """
    err = "cliches.write_good"
    msg = u"'{}' is a cliché."

    cliches = [
        "a chip off the old block",
        "a clean slate",
        "a dark and stormy night",
        "a far cry",
        "a fine kettle of fish",
        "a loose cannon",
        "a penny saved is a penny earned",
        "a tough row to hoe",
        "a word to the wise",
        "ace in the hole",
        "acid test",
        "add insult to injury",
        "against all odds",
        "air your dirty laundry",
        "all fun and games",
        "all in a day's work",
        "all talk, no action",
        "all thumbs",
        "all your eggs in one basket",
        "all's fair in love and war",
        "all's well that ends well",
        "almighty dollar",
        "American as apple pie",
        "an axe to grind",
        "another day, another dollar",
        "armed to the teeth",
        "as luck would have it",
        "as old as time",
        "as the crow flies",
        "at loose ends",
        "at my wits end",
        "avoid like the plague",
        "babe in the woods",
        "back against the wall",
        "back in the saddle",
        "back to square one",
        "back to the drawing board",
        "bad to the bone",
        "badge of honor",
        "bald faced liar",
        "ballpark figure",
        "banging your head against a brick wall",
        "baptism by fire",
        "barking up the wrong tree",
        "bat out of hell",
        "be all and end all",
        "beat a dead horse",
        "beat around the bush",
        "been there, done that",
        "beggars can't be choosers",
        "behind the eight ball",
        "bend over backwards",
        "benefit of the doubt",
        "bent out of shape",
        "best thing since sliced bread",
        "bet your bottom dollar",
        "better half",
        "better late than never",
        "better mousetrap",
        "better safe than sorry",
        "between a rock and a hard place",
        "beyond the pale",
        "bide your time",
        "big as life",
        "big cheese",
        "big fish in a small pond",
        "big man on campus",
        "bigger they are the harder they fall",
        "bird in the hand",
        "bird's eye view",
        "birds and the bees",
        "birds of a feather flock together",
        "bit the hand that feeds you",
        "bite the bullet",
        "bite the dust",
        "bitten off more than he can chew",
        "black as coal",
        "black as pitch",
        "black as the ace of spades",
        "blast from the past",
        "bleeding heart",
        "blessing in disguise",
        "blind ambition",
        "blind as a bat",
        "blind leading the blind",
        "blood is thicker than water",
        "blood sweat and tears",
        "blow off steam",
        "blow your own horn",
        "blushing bride",
        "boils down to",
        "bolt from the blue",
        "bone to pick",
        "bored stiff",
        "bored to tears",
        "bottomless pit",
        "boys will be boys",
        "bright and early",
        "brings home the bacon",
        "broad across the beam",
        "broken record",
        "brought back to reality",
        "bull by the horns",
        "bull in a china shop",
        "burn the midnight oil",
        "burning question",
        "burning the candle at both ends",
        "burst your bubble",
        "bury the hatchet",
        "busy as a bee",
        "by hook or by crook",
        "call a spade a spade",
        "called onto the carpet",
        "calm before the storm",
        "can of worms",
        "can't cut the mustard",
        "can't hold a candle to",
        "case of mistaken identity",
        "cat got your tongue",
        "cat's meow",
        "caught in the crossfire",
        "caught red-handed",
        "checkered past",
        "chomping at the bit",
        "cleanliness is next to godliness",
        "clear as a bell",
        "clear as mud",
        "close to the vest",
        "cock and bull story",
        "cold shoulder",
        "come hell or high water",
        "cool as a cucumber",
        "cool, calm, and collected",
        "cost a king's ransom",
        "count your blessings",
        "crack of dawn",
        "crash course",
        "creature comforts",
        "cross that bridge when you come to it",
        "crushing blow",
        "cry like a baby",
        "cry me a river",
        "cry over spilt milk",
        "crystal clear",
        "curiosity killed the cat",
        "cut and dried",
        "cut through the red tape",
        "cut to the chase",
        "cute as a bugs ear",
        "cute as a button",
        "cute as a puppy",
        "cuts to the quick",
        "dark before the dawn",
        "day in, day out",
        "dead as a doornail",
        "devil is in the details",
        "dime a dozen",
        "divide and conquer",
        "dog and pony show",
        "dog days",
        "dog eat dog",
        "dog tired",
        "don't burn your bridges",
        "don't count your chickens",
        "don't look a gift horse in the mouth",
        "don't rock the boat",
        "don't step on anyone's toes",
        "don't take any wooden nickels",
        "down and out",
        "down at the heels",
        "down in the dumps",
        "down the hatch",
        "down to earth",
        "draw the line",
        "dressed to kill",
        "dressed to the nines",
        "drives me up the wall",
        "dull as dishwater",
        "dyed in the wool",
        "eagle eye",
        "ear to the ground",
        "early bird catches the worm",
        "easier said than done",
        "easy as pie",
        "eat your heart out",
        "eat your words",
        "eleventh hour",
        "even the playing field",
        "every dog has its day",
        "every fiber of my being",
        "everything but the kitchen sink",
        "eye for an eye",
        "face the music",
        "facts of life",
        "fair weather friend",
        "fall by the wayside",
        "fan the flames",
        "feast or famine",
        "feather your nest",
        "feathered friends",
        "few and far between",
        "fifteen minutes of fame",
        "filthy vermin",
        "fine kettle of fish",
        "fish out of water",
        "fishing for a compliment",
        "fit as a fiddle",
        "fit the bill",
        "fit to be tied",
        "flash in the pan",
        "flat as a pancake",
        "flip your lid",
        "flog a dead horse",
        "fly by night",
        "fly the coop",
        "follow your heart",
        "for all intents and purposes",
        "for the birds",
        "for what it's worth",
        "force of nature",
        "force to be reckoned with",
        "forgive and forget",
        "fox in the henhouse",
        "free and easy",
        "free as a bird",
        "fresh as a daisy",
        "full steam ahead",
        "fun in the sun",
        "garbage in, garbage out",
        "gentle as a lamb",
        "get a kick out of",
        "get a leg up",
        "get down and dirty",
        "get the lead out",
        "get to the bottom of",
        "get your feet wet",
        "gets my goat",
        "gilding the lily",
        "give and take",
        "go against the grain",
        "go at it tooth and nail",
        "go for broke",
        "go him one better",
        "go the extra mile",
        "go with the flow",
        "goes without saying",
        "good as gold",
        "good deed for the day",
        "good things come to those who wait",
        "good time was had by all",
        "good times were had by all",
        "greased lightning",
        "greek to me",
        "green thumb",
        "green-eyed monster",
        "grist for the mill",
        "growing like a weed",
        "hair of the dog",
        "hand to mouth",
        "happy as a clam",
        "happy as a lark",
        "hasn't a clue",
        "have a nice day",
        "have high hopes",
        "have the last laugh",
        "haven't got a row to hoe",
        "head honcho",
        "head over heels",
        "hear a pin drop",
        "heard it through the grapevine",
        "heart's content",
        "heavy as lead",
        "hem and haw",
        "high and dry",
        "high and mighty",
        "high as a kite",
        "hit paydirt",
        "hold your head up high",
        "hold your horses",
        "hold your own",
        "hold your tongue",
        "honest as the day is long",
        "horns of a dilemma",
        "horse of a different color",
        "hot under the collar",
        "hour of need",
        "I beg to differ",
        "icing on the cake",
        "if the shoe fits",
        "if the shoe were on the other foot",
        "in a jam",
        "in a jiffy",
        "in a nutshell",
        "in a pig's eye",
        "in a pinch",
        "in a word",
        "in hot water",
        "in the gutter",
        "in the nick of time",
        "in the thick of it",
        "in your dreams",
        "it ain't over till the fat lady sings",
        "it goes without saying",
        "it takes all kinds",
        "it takes one to know one",
        "it's a small world",
        "it's only a matter of time",
        "ivory tower",
        "Jack of all trades",
        "jockey for position",
        "jog your memory",
        "joined at the hip",
        "judge a book by its cover",
        "jump down your throat",
        "jump in with both feet",
        "jump on the bandwagon",
        "jump the gun",
        "jump to conclusions",
        "just a hop, skip, and a jump",
        "just the ticket",
        "justice is blind",
        "keep a stiff upper lip",
        "keep an eye on",
        "keep it simple, stupid",
        "keep the home fires burning",
        "keep up with the Joneses",
        "keep your chin up",
        "keep your fingers crossed",
        "kick the bucket",
        "kick up your heels",
        "kick your feet up",
        "kid in a candy store",
        "kill two birds with one stone",
        "kiss of death",
        "knock it out of the park",
        "knock on wood",
        "knock your socks off",
        "know him from Adam",
        "know the ropes",
        "know the score",
        "knuckle down",
        "knuckle sandwich",
        "knuckle under",
        "labor of love",
        "ladder of success",
        "land on your feet",
        "lap of luxury",
        "last but not least",
        "last hurrah",
        "last-ditch effort",
        "law of the jungle",
        "law of the land",
        "lay down the law",
        "leaps and bounds",
        "let sleeping dogs lie",
        "let the cat out of the bag",
        "let the good times roll",
        "let your hair down",
        "let's talk turkey",
        "letter perfect",
        "lick your wounds",
        "lies like a rug",
        "life's a bitch",
        "life's a grind",
        "light at the end of the tunnel",
        "lighter than a feather",
        "lighter than air",
        "like clockwork",
        "like father like son",
        "like taking candy from a baby",
        "like there's no tomorrow",
        "lion's share",
        "live and learn",
        "live and let live",
        "long and short of it",
        "long lost love",
        "look before you leap",
        "look down your nose",
        "look what the cat dragged in",
        "looking a gift horse in the mouth",
        "looks like death warmed over",
        "loose cannon",
        "lose your head",
        "lose your temper",
        "loud as a horn",
        "lounge lizard",
        "loved and lost",
        "low man on the totem pole",
        "luck of the draw",
        "luck of the Irish",
        "make hay while the sun shines",
        "make money hand over fist",
        "make my day",
        "make the best of a bad situation",
        "make the best of it",
        "make your blood boil",
        "man of few words",
        "man's best friend",
        "mark my words",
        "meaningful dialogue",
        "missed the boat on that one",
        "moment in the sun",
        "moment of glory",
        "moment of truth",
        "money to burn",
        "more power to you",
        "more than one way to skin a cat",
        "movers and shakers",
        "moving experience",
        "naked as a jaybird",
        "naked truth",
        "neat as a pin",
        "needle in a haystack",
        "needless to say",
        "neither here nor there",
        "never look back",
        "never say never",
        "nip and tuck",
        "nip it in the bud",
        "no guts, no glory",
        "no love lost",
        "no pain, no gain",
        "no skin off my back",
        "no stone unturned",
        "no time like the present",
        "no use crying over spilled milk",
        "nose to the grindstone",
        "not a hope in hell",
        "not a minute's peace",
        "not in my backyard",
        "not playing with a full deck",
        "not the end of the world",
        "not written in stone",
        "nothing to sneeze at",
        "nothing ventured nothing gained",
        "now we're cooking",
        "off the top of my head",
        "off the wagon",
        "off the wall",
        "old hat",
        "older and wiser",
        "older than dirt",
        "older than Methuselah",
        "on a roll",
        "on cloud nine",
        "on pins and needles",
        "on the bandwagon",
        "on the money",
        "on the nose",
        "on the rocks",
        "on the spot",
        "on the tip of my tongue",
        "on the wagon",
        "on thin ice",
        "once bitten, twice shy",
        "one bad apple doesn't spoil the bushel",
        "one born every minute",
        "one brick short",
        "one foot in the grave",
        "one in a million",
        "one red cent",
        "only game in town",
        "open a can of worms",
        "open and shut case",
        "open the flood gates",
        "opportunity doesn't knock twice",
        "out of pocket",
        "out of sight, out of mind",
        "out of the frying pan into the fire",
        "out of the woods",
        "out on a limb",
        "over a barrel",
        "over the hump",
        "pain and suffering",
        "pain in the",
        "panic button",
        "par for the course",
        "part and parcel",
        "party pooper",
        "pass the buck",
        "patience is a virtue",
        "pay through the nose",
        "penny pincher",
        "perfect storm",
        "pig in a poke",
        "pile it on",
        "pillar of the community",
        "pin your hopes on",
        "pitter patter of little feet",
        "plain as day",
        "plain as the nose on your face",
        "play by the rules",
        "play your cards right",
        "playing the field",
        "playing with fire",
        "pleased as punch",
        "plenty of fish in the sea",
        "point with pride",
        "poor as a church mouse",
        "pot calling the kettle black",
        "pretty as a picture",
        "pull a fast one",
        "pull your punches",
        "pulling your leg",
        "pure as the driven snow",
        "put it in a nutshell",
        "put one over on you",
        "put the cart before the horse",
        "put the pedal to the metal",
        "put your best foot forward",
        "put your foot down",
        "quick as a bunny",
        "quick as a lick",
        "quick as a wink",
        "quick as lightning",
        "quiet as a dormouse",
        "rags to riches",
        "raining buckets",
        "raining cats and dogs",
        "rank and file",
        "rat race",
        "reap what you sow",
        "red as a beet",
        "red herring",
        "reinvent the wheel",
        "rich and famous",
        "rings a bell",
        "ripe old age",
        "ripped me off",
        "rise and shine",
        "road to hell is paved with good intentions",
        "rob Peter to pay Paul",
        "roll over in the grave",
        "rub the wrong way",
        "ruled the roost",
        "running in circles",
        "sad but true",
        "sadder but wiser",
        "salt of the earth",
        "scared stiff",
        "scared to death",
        "sealed with a kiss",
        "second to none",
        "see eye to eye",
        "seen the light",
        "seize the day",
        "set the record straight",
        "set the world on fire",
        "set your teeth on edge",
        "sharp as a tack",
        "shoot for the moon",
        "shoot the breeze",
        "shot in the dark",
        "shoulder to the wheel",
        "sick as a dog",
        "sigh of relief",
        "signed, sealed, and delivered",
        "sink or swim",
        "six of one, half a dozen of another",
        "skating on thin ice",
        "slept like a log",
        "slinging mud",
        "slippery as an eel",
        "slow as molasses",
        "smart as a whip",
        "smooth as a baby's bottom",
        "sneaking suspicion",
        "snug as a bug in a rug",
        "sow wild oats",
        "spare the rod, spoil the child",
        "speak of the devil",
        "spilled the beans",
        "spinning your wheels",
        "spitting image of",
        "spoke with relish",
        "spread like wildfire",
        "spring to life",
        "squeaky wheel gets the grease",
        "stands out like a sore thumb",
        "start from scratch",
        "stick in the mud",
        "still waters run deep",
        "stitch in time",
        "stop and smell the roses",
        "straight as an arrow",
        "straw that broke the camel's back",
        "strong as an ox",
        "stubborn as a mule",
        "stuff that dreams are made of",
        "stuffed shirt",
        "sweating blood",
        "sweating bullets",
        "take a load off",
        "take one for the team",
        "take the bait",
        "take the bull by the horns",
        "take the plunge",
        "takes one to know one",
        "takes two to tango",
        "the more the merrier",
        "the real deal",
        "the real McCoy",
        "the red carpet treatment",
        "the same old story",
        "there is no accounting for taste",
        "thick as a brick",
        "thick as thieves",
        "thin as a rail",
        "think outside of the box",
        "third time's the charm",
        "this day and age",
        "this hurts me worse than it hurts you",
        "this point in time",
        "three sheets to the wind",
        "through thick and thin",
        "throw in the towel",
        "tie one on",
        "tighter than a drum",
        "time and time again",
        "time is of the essence",
        "tip of the iceberg",
        "tired but happy",
        "to coin a phrase",
        "to each his own",
        "to make a long story short",
        "to the best of my knowledge",
        "toe the line",
        "tongue in cheek",
        "too good to be true",
        "too hot to handle",
        "too numerous to mention",
        "touch with a ten foot pole",
        "tough as nails",
        "trial and error",
        "trials and tribulations",
        "tried and true",
        "trip down memory lane",
        "twist of fate",
        "two cents worth",
        "two peas in a pod",
        "ugly as sin",
        "under the counter",
        "under the gun",
        "under the same roof",
        "under the weather",
        "until the cows come home",
        "unvarnished truth",
        "up the creek",
        "uphill battle",
        "upper crust",
        "upset the applecart",
        "vain attempt",
        "vain effort",
        "vanquish the enemy",
        "vested interest",
        "waiting for the other shoe to drop",
        "wakeup call",
        "warm welcome",
        "watch your p's and q's",
        "watch your tongue",
        "watching the clock",
        "water under the bridge",
        "weather the storm",
        "weed them out",
        "week of Sundays",
        "went belly up",
        "wet behind the ears",
        "what goes around comes around",
        "what you see is what you get",
        "when it rains, it pours",
        "when push comes to shove",
        "when the cat's away",
        "when the going gets tough, the tough get going",
        "white as a sheet",
        "whole ball of wax",
        "whole hog",
        "whole nine yards",
        "wild goose chase",
        "will wonders never cease?",
        "wisdom of the ages",
        "wise as an owl",
        "wolf at the door",
        "words fail me",
        "work like a dog",
        "world weary",
        "worst nightmare",
        "worth its weight in gold",
        "wrong side of the bed",
        "yanking your chain",
        "yappy as a dog",
        "years young",
        "you are what you eat",
        "you can run but you can't hide",
        "you only live once",
        "you're the boss ",
        "young and foolish",
        "young and vibrant",
    ]

    return existence_check(text, cliches, err, msg, join=True)


@memoize
def check_cliches_gnu_diction(text):
    """Check the text.

    source:     GNU diction
    source_url: https://directory.fsf.org/wiki/Diction
    """
    err = "cliches.gnu_diction"
    msg = u"'{}' is a cliché."

    list = [
        "a matter of concern",
        "all things being equal",
        "as a last resort",
        "attached hereto",
        "by no means",
        "conspicuous by its absence",
        "easier said than done",
        "enclosed herewith",
        "if and when",
        "in reference to",
        "in short supply",
        "in the foreseeable future",
        "in the long run",
        "in the matter of",
        "it stands to reason",
        "many and diverse",
        "on the right track",
        "par for the course",
        "please feel free to",
        "pursuant to your request",
        "regarding the matter of",
        "slowly but surely",
        "this will acknowledge",
        "we are pleased to advice",
        "we regret to inform you",
        "we wish to state",
        "you are hereby advised that",
    ]

    return existence_check(text, list, err, msg, join=True, ignore_case=True)



# -*- coding: utf-8 -*-
"""Very.

---
layout:     post
source:     William Allen White
source_url: http://bit.ly/1XaMllw
title:      very
date:       2014-06-10 12:31:19
categories: writing
---

Substitute 'damn' every time you're inclined to write 'very;' your editor will
delete it and the writing will be just as it should be.

"""
from proselint.tools import memoize, existence_check


@memoize
def check_very_damn(text):
    """Avoid 'very'."""
    err = "weasel_words.very"
    msg = ("Substitute 'damn' every time you're "
           "inclined to write 'very;' your editor will delete it "
           "and the writing will be just as it should be.")
    regex = "very"

    return existence_check(text, [regex], err, msg, max_errors=1)



# -*- coding: utf-8 -*-
u"""Don't start a paragraph with 'But'.

---
layout:
source: Justin Jungé
source_url:
title:
date:       2016-03-10 12:31:19
categories: writing
---

Paragraphs should not start with certain bad words.

"""
from proselint.tools import memoize, existence_check


@memoize
def check_dont_start_with_buts(text):
    """Do not start a paragraph with a 'But'."""
    err = "misc.but"
    msg = u"No paragraph should start with a 'But'."
    regex = "(^|([\n\r]+))(\s*)But"
    return existence_check(text, [regex], err, msg)



"""Test version number."""
from __future__ import absolute_import
from __future__ import print_function

from .check import Check
from proselint.version import __version__
import subprocess


class TestCheck(Check):
    """Test class for version number."""

    __test__ = True

    def test_version(self):
        """Make sure the version number is correct."""
        out = subprocess.check_output("proselint --version", shell=True)
        assert out.decode('utf-8') == __version__ + "\n"



# -*- coding: utf-8 -*-
u"""Many a singular.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      Many a singular.
date:       2014-06-10 12:31:19
categories: writing
---

The idiom 'many a' requires a singular verb.

"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "misc.many_a"
    msg = "'many a' requires a singular verb."

    preferences = [
        ["is many a",          ["are many a"]],
        ["has been many a",    ["have been many a"]],
        ["was many a",         ["were many a"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



# -*- coding: utf-8 -*-

"""-able vs. -ible."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check_able_ible(text):
    """-able vs. -ible."""
    err = "spelling.able_ible"
    msg = "-able vs. -ible. '{}' is the preferred spelling."

    preferences = [

        ["actionable",      ["actionible"]],
        ["addable",         ["addible"]],
        ["admittable",      ["admittible"]],
        ["advisable",       ["advisible"]],
        ["affectable",      ["affectible"]],
        ["allegeable",      ["allegeible"]],
        ["analyzable",      ["analyzible"]],
        ["annexable",       ["annexible"]],
        ["arrestable",      ["arrestible"]],
        ["ascendable",      ["ascendible"]],
        ["assertable",      ["assertible"]],
        ["assessable",      ["assessible"]],
        ["averageable",     ["averageible"]],
        ["bailable",        ["bailible"]],
        ["blamable",        ["blamible"]],
        ["changeable",      ["changeible"]],
        ["chargeable",      ["chargeible"]],
        ["circumscribable", ["circumscribible"]],
        ["commensurable",   ["commensurible"]],
        ["committable",     ["committible"]],
        ["condensable",     ["condensible"]],
        ["connectable",     ["connectible"]],
        ["contestable",     ["contestible"]],
        ["contractable",    ["contractible"]],
        ["conversable",     ["conversible"]],
        ["convictable",     ["convictible"]],
        ["correctable",     ["correctible"]],
        ["definable",       ["definible"]],
        ["detectable",      ["detectible"]],
        ["diagnosable",     ["diagnosible"]],
        ["discussable",     ["discussible"]],
        ["endorsable",      ["endorsible"]],
        ["enforceable",     ["enforceible"]],
        ["evadable",        ["evadible"]],
        ["excisable",       ["excisible"]],
        ["excludable",      ["excludible"]],
        ["expandable",      ["expandible"]],
        ["extendable",      ["extendible"]],
        ["extractable",     ["extractible"]],
        ["ignitable",       ["ignitible"]],
        ["immovable",       ["immovible"]],
        ["improvable",      ["improvible"]],
        ["includable",      ["includible"]],
        ["inferable",       ["inferible"]],
        ["inventable",      ["inventible"]],
        ["investable",      ["investible"]],
        ["lapsable",        ["lapsible"]],
        ["lovable",         ["lovible"]],
        ["mixable",         ["mixible"]],
        ["movable",         ["movible"]],
        ["noticeable",      ["noticeible"]],
        ["offendable",      ["offendible"]],
        ["patentable",      ["patentible"]],
        ["persuadable",     ["persuadible"]],
        ["preventable",     ["preventible"]],
        ["processable",     ["processible"]],
        ["protectable",     ["protectible"]],
        ["ratable",         ["ratible"]],
        ["redressable",     ["redressible"]],
        ["referable",       ["referible"]],
        ["retractable",     ["retractible"]],
        ["revisable",       ["revisible"]],
        ["rinsable",        ["rinsible"]],
        ["salable",         ["salible"]],
        ["suspendable",     ["suspendible"]],
        ["tractable",       ["tractible"]],
        ["transferable",    ["transferible"]],
        ["transmittable",   ["transmittible"]],
        ["willable",        ["willible"]],

        ["accessible",      ["accessable"]],
        ["adducible",       ["adducable"]],
        ["admissible",      ["admissable"]],
        ["audible",         ["audable"]],
        ["avertible",       ["avertable"]],
        ["collapsible",     ["collapsable"]],
        ["collectible",     ["collectable"]],
        ["combustible",     ["combustable"]],
        ["compactible",     ["compactable"]],
        ["compatible",      ["compatable"]],
        ["comprehensible",  ["comprehensable"]],
        ["compressible",    ["compressable"]],
        ["concussible",     ["concussable"]],
        ["conductible",     ["conductable"]],
        ["contemptible",    ["contemptable"]],
        ["controvertible",  ["controvertable"]],
        ["convertible",     ["convertable"]],
        ["corrodible",      ["corrodable"]],
        ["corruptible",     ["corruptable"]],
        ["credible",        ["credable"]],
        ["deducible",       ["deducable"]],
        ["deductible",      ["deductable"]],
        ["defeasible",      ["defeasable"]],
        ["defensible",      ["defensable"]],
        ["descendible",     ["descendable"]],
        ["destructible",    ["destructable"]],
        ["diffusible",      ["diffusable"]],
        ["digestible",      ["digestable"]],
        ["discernible",     ["discernable"]],
        ["dismissible",     ["dismissable"]],
        ["divisible",       ["divisable"]],
        ["edible",          ["edable"]],
        ["educible",        ["educable"]],
        ["eligible",        ["eligable"]],
        ["erodible",        ["erodable"]],
        ["exhaustible",     ["exhaustable"]],
        ["expressible",     ["expressable"]],
        ["fallible",        ["fallable"]],
        ["feasible",        ["feasable"]],
        ["flexible",        ["flexable"]],
        ["forcible",        ["forcable"]],
        ["fusible",         ["fusable"]],
        ["gullible",        ["gullable"]],
        ["horrible",        ["horrable"]],
        ["impressible",     ["impressable"]],
        ["inadmissible",    ["inadmissable"]],
        ["incorrigible",    ["incorrigable"]],
        ["indelible",       ["indelable"]],
        ["inexpressible",   ["inexpressable"]],
        ["intelligible",    ["intelligable"]],
        ["interfusible",    ["interfusable"]],
        ["invincible",      ["invincable"]],
        ["irascible",       ["irascable"]],
        ["irresistible",    ["irresistable"]],
        ["legible",         ["legable"]],
        ["negligible",      ["negligable"]],
        ["omissible",       ["omissable"]],
        ["oppressible",     ["oppressable"]],
        ["ostensible",      ["ostensable"]],
        ["perceptible",     ["perceptable"]],
        ["perfectible",     ["perfectable"]],
        ["permissible",     ["permissable"]],
        ["plausible",       ["plausable"]],
        ["possible",        ["possable"]],
        ["producible",      ["producable"]],
        ["reducible",       ["reducable"]],
        ["remissible",      ["remissable"]],
        ["reprehensible",   ["reprehensable"]],
        ["repressible",     ["repressable"]],
        ["resistible",      ["resistable"]],
        ["responsible",     ["responsable"]],
        ["reversible",      ["reversable"]],
        ["revertible",      ["revertable"]],
        ["risible",         ["risable"]],
        ["seducible",       ["seducable"]],
        ["sensible",        ["sensable"]],
        ["submersible",     ["submersable"]],
        ["submergible",     ["submergable"]],
        ["suggestible",     ["suggestable"]],
        ["suppressible",    ["suppressable"]],
        ["susceptible",     ["susceptable"]],
        ["terrible",        ["terrable"]],
        ["transfusible",    ["transfusable"]],
        ["transmissible",   ["transmissable"]],
        ["uncollectible",   ["uncollectable"]],
        ["vendible",        ["vendable"]],
        ["visible",         ["visable"]]
    ]

    return preferred_forms_check(text, preferences, err, msg)



# -*- coding: utf-8 -*-
"""Punctuation.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      dates
date:       2014-06-10 12:31:19
categories: writing
---

Dates.

"""
from proselint.tools import existence_check, memoize


@memoize
def check_et_al(text):
    """Check the text."""
    err = "garner.punctuation"
    msg = u"Misplaced punctuation. It's 'et al.'"

    list = [
        "et. al",
        "et. al."
    ]
    return existence_check(text, list, err, msg, join=True)



# -*- coding: utf-8 -*-
"""Illogic.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      Illogic
date:       2014-06-10 12:31:19
categories: writing
---

Archaism.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "misc.illogic"
    msg = u"'{}' is illogical."

    illogics = [
        "preplan",
        "more than .{1,10} all",
        "appraisal valuations?",
        "(?:i|you|he|she|it|y'all|all y'all|you all|they) could care less",
        "least worst",
        "much-needed gaps?",
        "much-needed voids?",
        "no longer requires oxygen",
        "without scarcely",
    ]

    return existence_check(text, illogics, err, msg, offset=1)


@memoize
def check_coin_a_phrase_from(text):
    """Check the text."""
    err = "misc.illogic.coin"
    msg = "You can't coin an existing phrase. Did you mean 'borrow'?"

    regex = "to coin a phrase from"

    return existence_check(text, [regex], err, msg, offset=1)


@memoize
def check_without_your_collusion(text):
    """Check the textself."""
    err = "misc.illogic.collusion"
    msg = "It's impossible to defraud yourself. Try 'aquiescence'."

    regex = "without your collusion"

    return existence_check(
        text, [regex], err, msg, require_padding=False, offset=-1)



"""Test the topic detector tool."""

from proselint.tools import topics


def test_50_Cent_detector_on_topic():
    """Check precision of the 50 Cent topic-detector."""
    text = """With the aid of Eminem and Dr. Dre (who produced his first
           major-label album, Get Rich or Die Tryin'), Jackson became one
           of the world's best selling rappers and rose to prominence with
           East Coast hip hop group G-Unit (which he leads de facto). """

    assert("50 Cent" in topics(text))

    text = """Hip hop was started in the early 50's."""
    assert("50 Cent" not in topics(text))

    text = """Nowadays it costs 50 cents to buy a lollipop."""
    assert("50 Cent" not in topics(text))



# -*- coding: utf-8 -*-
"""Misspellings.

---
layout:     post
source:     The Wall Street Journal
source_url: http://on.wsj.com/1rksm8k
title:      misspellings
date:       2014-06-10 12:31:19
categories: writing
---

Points out misspellings.

"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "spelling.athletes"
    msg = "Misspelling of athlete's name. '{}' is the preferred form."

    misspellings = [
        ["Dwyane Wade",         ["Dwayne Wade"]],
        ["Miikka Kiprusoff",    ["Mikka Kiprusoff"]],
        ["Mark Buehrle",        ["Mark Buerhle"]],
        ["Skylar Diggins",      ["Skyler Diggins"]],
        ["Agnieszka Radwanska", ["Agnieska Radwanska"]],
        ["J.J. Redick",         ["J.J. Reddick"]],
        ["Manny Pacquiao",      ["Manny Packquaio"]],
        ["Antawn Jamison",      ["Antwan Jamison"]],
        ["Cal Ripken",          ["Cal Ripkin"]],
        ["Jhonny Peralta",      ["Johnny Peralta"]],
        ["Monta Ellis",         ["Monte Ellis"]],
        ["Alex Rodriguez",      ["Alex Rodriquez"]],
        ["Mark Teixeira",       ["Mark Texeira"]],
        ["Brett Favre",         ["Brett Farve"]],
        ["Torii Hunter",        ["Tori Hunter"]],
        ["Stephen Curry",       ["Stephon Curry"]],
        ["Mike Krzyzewski",     ["Mike Kryzewski"]],
    ]

    return preferred_forms_check(text, misspellings, err, msg)



"""Check that a check is working."""

from past.builtins import basestring
from unittest import TestCase
import os
import codecs


class Check(TestCase):
    """All tests inherit from Check."""

    __test__ = False

    def setUp(self):
        """Placeholder for setup procedure."""
        pass

    def tearDown(self):
        """Placeholder for teardown procedure."""
        from proselint.tools import close_cache_shelves
        close_cache_shelves()

    @property
    def this_check(self):
        """The specific check."""
        raise NotImplementedError

    def passes(self, lst):
        """Check if the test runs cleanly on the given text."""
        if isinstance(lst, basestring):
            lst = [lst]

        errors = []
        for text in lst:
            errors.append(self.this_check.check(text))

        return len(errors[0]) == 0

    def wpe_too_high(self):
        """Check whether the check is too noisy."""
        min_wpe = 50

        examples_dir = os.path.join(os.getcwd(), "tests", "corpus")
        examples = os.listdir(examples_dir)

        for example in examples:
            example_path = os.path.join(examples_dir, example)

            if ".DS_Store" in example_path:
                break

            # Compute the number of words per (wpe) error.
            with codecs.open(example_path, "r", encoding='utf-8') as f:
                text = f.read()
                num_errors = len(self.this_check.check(text))
                num_words = len(text)

            try:
                wpe = 1.0 * num_words / num_errors
            except ZeroDivisionError:
                wpe = float('Inf')

            # Make sure that
            assert wpe > min_wpe, \
                "{} has only {} wpe.".format(example, round(wpe, 2))



"""Test junge.but."""
from __future__ import absolute_import
from __future__ import print_function

from .check import Check
from proselint.checks.misc import but

from nose.tools import assert_equals


class TestCheck(Check):
    """Test class for paragraph."""

    __test__ = True

    def test_starting_with_But_is_error(self):
        """Starting with but should be an error."""
        text = 'But I don\'t know what to say?'
        errors = but.check_dont_start_with_buts(text)
        assert_equals(len(errors), 1)

    def test_starting_with_lowercase_but_is_error(self):
        """Starting with lowercase but should be an error."""
        text = 'but I don\'t know what to say?'
        errors = but.check_dont_start_with_buts(text)
        assert_equals(len(errors), 1)

    def test_starts_with_valid_word(self):
        """Starting with a valid word is not an error."""
        text = 'Hello.'
        errors = but.check_dont_start_with_buts(text)
        assert_equals(len(errors), 0)

    def test_butcher_is_ok(self):
        """Butcher is not an error, even though it has a But."""
        text = 'Butchers are ok.'
        errors = but.check_dont_start_with_buts(text)
        assert_equals(len(errors), 0)

    def test_but_in_middle_of_paragraph(self):
        """But in the middle of a sentance is ok."""
        text = 'I\'ve got nothing but nice things to say.'
        errors = but.check_dont_start_with_buts(text)
        assert_equals(len(errors), 0)

    def test_but_after_a_newline(self):
        """But after a newline should be an error."""
        text = '\nBut what more.'
        errors = but.check_dont_start_with_buts(text)
        assert_equals(len(errors), 1)

    def test_but_after_a_newline_and_whitespace(self):
        """But after a newline and whitespace should be an error."""
        text = '\n But what more.'
        errors = but.check_dont_start_with_buts(text)
        assert_equals(len(errors), 1)



"""Test Butterick's symbols."""
from __future__ import absolute_import

from .check import Check

from proselint.checks.typography import symbols as chk


class TestCheck(Check):
    """The test class for typography.symbols."""

    __test__ = True

    @property
    def this_check(self):
        """Bolierplate."""
        return chk

    def test_ellipsis(self):
        """Find ... in a string."""
        assert chk.check_ellipsis("""The long and winding road...""")

    def test_copyright(self):
        """Find a (c) or (C) in a string."""
        assert chk.check_copyright_symbol("""Show me the money! (C)""")
        assert chk.check_copyright_symbol("""Show me the money! (c)""")

    def test_trademark(self):
        """Find a (TM) or (tm) in a string."""
        assert chk.check_trademark_symbol("""The Fresh Maker (TM)""")
        assert chk.check_trademark_symbol("""The Fresh Maker (tm)""")

    def test_registered_trademark(self):
        """Find a (r) or (R) in a string."""
        assert chk.check_registered_trademark_symbol("""Just Do It (R)""")
        assert chk.check_registered_trademark_symbol("""Just Do It (r)""")

    def test_sentence_spacing(self):
        """Find a sentence followed by three or more spaces."""
        assert chk.check_sentence_spacing(
            """This is a sentence.   This is another.""")

    def test_multiplication(self):
        """Find an x between two digits."""
        assert chk.check_multiplication_symbol(
            """It is obvious that 2 x 2 = 4.""")

    def test_curly_quotes(self):  # FIXME
        """Find "" quotes in a string."""
        pass



# -*- coding: utf-8 -*-
"""Oxymorons.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      Oxymorons
date:       2014-06-10 12:31:19
categories: writing
---

Archaism.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "oxymorons.misc"
    msg = u"'{}' is an oxymoron."

    oxymorons = [
        "amateur expert",
        "increasingly less",
        "advancing backwards?",
        "alludes explicitly to",
        "explicitly alludes to",
        "totally obsolescent",
        "completely obsolescent",
        "generally always",
        "usually always",
        "increasingly less",
        "build down",
        "conspicuous absence",
        "exact estimate",
        "executive secretary",
        "found missing",
        "intense apathy",
        "mandatory choice",
        "nonworking mother",
        "organized mess",
        # "pretty ugly",
        # "sure bet",
    ]

    return existence_check(text, oxymorons, err, msg, offset=1, join=True)



# -*- coding: utf-8 -*-
"""Diacritical marks.

Use of diacritical marks where common.
"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "typography.diacritical_marks"
    msg = u"Use diacritical marks in '{}'."

    list = [
        # French loanwords
        [u"beau idéal",         ["beau ideal"]],
        [u"boutonnière",        ["boutonniere"]],
        [u"bric-à-brac",        ["bric-a-brac"]],
        [u"café",               ["cafe"]],
        [u"cause célèbre",      ["cause celebre"]],
        [u"chèvre",             ["chevre"]],
        [u"cliché",             ["cliche"]],
        [u"comme ci comme ça",  ["comme ci comme ca", "comsi comsa"]],
        [u"consommé",           ["consomme"]],
        [u"coup d'état",        ["coup d'etat"]],
        [u"coup de grâce",      ["coup de grace"]],
        [u"crudités",           ["crudites"]],
        [u"crème brûlée",       ["creme brulee"]],
        [u"crème de menthe",    ["creme de menthe"]],
        [u"crème fraîche",      ["creme fraice"]],
        [u"crème fraîche",      ["creme fresh"]],
        [u"crêpe",              ["crepe"]],
        [u"débutante",          ["debutante"]],
        [u"décor",              ["decor"]],
        [u"déjà vu",            ["deja vu"]],
        [u"dénouement",         ["denouement"]],
        [u"façade",             ["facade"]],
        [u"fiancé",             ["fiance"]],
        [u"fiancée",            ["fiancee"]],
        [u"flambé",             ["flambe"]],
        [u"garçon",             ["garcon"]],
        [u"lycée",              ["lycee"]],
        [u"maître d",           ["maitre d"]],
        [u"ménage à trois",     ["menage a trois"]],
        [u"négligée",           ["negligee"]],
        [u"papier-mâché", ["papier-mache", "paper mache", "paper-mache"]],
        [u"protégé",            ["protege"]],
        [u"protégée",           ["protegee"]],
        [u"purée",              ["puree"]],
        [u"raison d'être",      ["raison d'etre"]],
        [u"my résumé",          ["my resume"]],
        [u"your résumé",        ["your resume"]],
        [u"his résumé",         ["his resume"]],
        [u"her résumé",         ["her resume"]],
        [u"a résumé",           ["a resume"]],
        [u"the résumé",         ["the resume"]],
        [u"risqué",             ["risque"]],
        [u"roué",               ["roue"]],
        [u"soirée",             ["soiree"]],
        [u"soufflé",            ["souffle"]],
        [u"soupçon",            ["soupcon"]],
        [u"touché",             ["touche"]],
        [u"tête-à-tête",        ["tete-a-tete"]],
        [u"voilà",              ["voila"]],
        [u"à la carte",         ["a la carte"]],
        [u"à la mode",          ["a la mode"]],
        [u"émigré",             ["emigre"]],

        # Spanish loanwords
        [u"El Niño",            ["El Nino"]],
        [u"jalapeño",           ["jalapeno"]],
        [u"La Niña",            ["La Nina"]],
        [u"piña colada",        ["pina colada"]],
        [u"señor",              ["senor"]],
        [u"señora",             ["senora"]],
        [u"señorita",           ["senorita"]],

        # Portuguese loanwords
        [u"açaí",               ["acai"]],

        # German loanwords
        [u"doppelgänger",       ["doppelganger"]],
        [u"Führer",             ["Fuhrer"]],
        [u"Gewürztraminer",     ["Gewurztraminer"]],
        [u"vis-à-vis",          ["vis-a-vis"]],
        [u"Übermensch",         ["Ubermensch"]],

        # Swedish loandwords
        [u"filmjölk",           ["filmjolk"]],
        [u"smörgåsbord",        ["smorgasbord"]],

        # Names, places, and companies
        [u"Beyoncé",            ["Beyonce"]],
        [u"Brontë",             ["Bronte"]],
        [u"Brontë",             ["Bronte"]],
        [u"Champs-Élysées",     ["Champs-Elysees"]],
        [u"Citroën",            ["Citroen"]],
        [u"Curaçao",            ["Curacao"]],
        [u"Häagen-Dazs",        ["Haagen-Dazs", "Haagen Dazs"]],
        [u"Löwenbräu",          ["Lowenbrau"]],
        [u"Monégasque",         ["Monegasque"]],
        [u"Mötley Crüe",        ["Motley Crue"]],
        [u"Nescafé",            ["Nescafe"]],
        [u"Queensrÿche",        ["Queensryche"]],
        [u"Québec",             ["Quebec"]],
        [u"Québécois",          ["Quebecois"]],
        [u"Ångström",           ["Angstrom"]],
        [u"ångström",           ["angstrom"]],
        [u"Škoda",              ["Skoda"]],
    ]

    return preferred_forms_check(text, list, err, msg)



"""Check that the CLI returns the appropriate exit code."""

import subprocess


def test_exit_code_demo():
    """Ensure that linting the demo returns an exit code of 1."""
    try:
        subprocess.check_output("proselint --demo", shell=True)

    except subprocess.CalledProcessError as grepexc:
        assert(grepexc.returncode == 1)


def test_exit_code_version():
    """Ensure that getting the version returns an exit code of 0."""
    try:
        subprocess.check_output("proselint --version", shell=True)

    except subprocess.CalledProcessError:
        assert(False)



# -*- coding: utf-8 -*-

"""Denizen labels."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check_denizen_labels_ganer(text):
    """Suggest the preferred forms.

    source:     Garner's Modern American Usage
    source_url: http://bit.ly/1T4alrY
    """
    err = "terms.denizen_labels.garner"
    msg = "'{}' is the preferred denizen label."

    preferences = [

        ["Afrikaner",         ["Afrikaaner"]],
        ["Afrikaner",         ["Afrikander"]],
        ["Alabamian",         ["Alabaman"]],
        ["Albuquerquean",     ["Albuquerquian"]],
        ["Anchorageite",      ["Anchoragite"]],
        ["Angeleno",          ["Los Angelean"]],
        ["Arizonan",          ["Arizonian"]],
        ["Arkansan",          ["Arkansawyer"]],
        ["Belarusian",        ["Belarusan"]],
        ["Caymanian",         ["Cayman Islander"]],
        ["Coloradan",         ["Coloradoan"]],
        ["Fairbanksan",       ["Fairbanksian"]],
        ["Fort Worthian",     ["Fort Worther"]],
        ["Grenadan",          ["Grenadian"]],
        ["Hong Konger",       ["Hong Kongite", "Hong Kongian"]],
        ["Hoosier",           ["Indianan", "Indianian"]],
        ["Illinoisan",        ["Illinoisian"]],
        ["Iowan",             ["Iowegian"]],
        ["Louisianian",       ["Louisianan"]],
        ["Michigander",       ["Michiganite", "Michiganian"]],
        ["Missourian",        ["Missouran"]],
        ["Monegasque",        ["Monacan"]],
        ["Neapolitan",        ["Neopolitan"]],
        ["New Hampshirite",   ["New Hampshireite", "New Hampshireman"]],
        ["New Jerseyan",      ["New Jerseyite"]],
        ["New Orleanian",     ["New Orleansian"]],
        ["Nutmegger",         ["Connecticuter"]],
        ["Oklahoma Cityan",   ["Oklahoma Citian"]],
        ["Oklahoman",         ["Oklahomian"]],
        ["Seattleite",        ["Seattlite"]],
        ["Surinamese",        ["Surinamer"]],
        ["Tallahasseean",     ["Tallahassean"]],
        ["Tennessean",        ["Tennesseean"]],
        ["Tusconan",          ["Tusconian", "Tusconite"]],
        ["Utahn",             ["Utahan"]],
        ["Saudi",             ["Saudi Arabian"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)


@memoize
def check_denizen_labels_norris(text):
    """Suggest the preferred forms.

    source:     Mary Norris
    source_url: http://nyr.kr/1rGienj
    """
    err = "terms.denizen_labels.norris"
    msg = "Would you like '{}'?"

    preferences = [
        ["Mancunian",         ["Manchesterian"]],
        ["Mancunians",        ["Manchesterians"]],
        ["Vallisoletano",     ["Valladolidian"]],
        ["Wulfrunian",        ["Wolverhamptonian", "Wolverhamptonite"]],
        ["Novocastrian",      ["Newcastleite", "Newcastlite"]],
        ["Trifluvian",        [u"Trois-Rivièrester"]],
        ["Leodenisian",       ["Leedsian"]],
        ["Minneapolitan",     ["Minneapolisian"]],
        ["Hartlepudlian",     ["Hartlepoolian"]],
        ["Liverpudlian",      ["Liverpoolian"]],
        ["Haligonian",        ["Halifaxer"]],
        ["Varsovian",         ["Warsawer", "Warsawian"]],
        ["Providentian",      ["Providencian", "Providencer"]],
        ["Tridentine",        ["Trentian", "Trentonian"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



# -*- coding: utf-8 -*-
"""Commercialese.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      commercialese
date:       2014-06-10 12:31:19
categories: writing
---

Commercialese.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "misc.commercialese"
    msg = u"'{}' is commercialese."

    commercialese = [
        "acknowledging yours of",
        "beg to advise",
        "enclosed herewith",
        "enclosed please find",
        "further to yours of",
        "further to your letter",
        "in regard to",
        "inst\.",
        "in the amount of",
        "of even date",
        "pending receipt of",
        "please be advised that",
        "please return same",
        "pleasure of a reply",
        "prox\.",
        "pursuant to your request",
        "regarding the matter",
        "regret to inform",
        "thanking you in advance",
        "the undersigned",
        "this acknowledges your letter",
        "ult\."
        "we are pleased to note",
        "with regard to",
        "your favor has come to hand",
        "yours of even date"
    ]

    return existence_check(text, commercialese, err, msg, join=True)



# -*- coding: utf-8 -*-
"""Dates.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      dates
date:       2014-06-10 12:31:19
categories: writing
---

Dates.

"""
from proselint.tools import existence_check, memoize
import calendar


@memoize
def check_decade_apostrophes_short(text):
    """Check the text for dates of the form X0's."""
    err = "dates_times.dates"
    msg = u"Apostrophes aren't needed for decades."

    regex = "\d0\'s"

    return existence_check(
        text, [regex], err, msg, excluded_topics=["50 Cent"])


@memoize
def check_decade_apostrophes_long(text):
    """Check the text for dates of the form XXX0's."""
    err = "dates_times.dates"
    msg = u"Apostrophes aren't needed for decades."

    regex = "\d\d\d0\'s"
    return existence_check(text, [regex], err, msg)


@memoize
def check_dash_and_from(text):
    """Check the text."""
    err = "dates_times.dates"
    msg = u"When specifying a date range, write 'from X to Y'."

    regex = "[fF]rom \d+[^ \t\n\r\f\va-zA-Z0-9_\.]\d+"
    return existence_check(text, [regex], err, msg)


def check_month_year_comma(text):
    """Check the text."""
    err = "dates_times.dates"
    msg = u"When specifying a month and year, no comma is needed."

    regex = "(?:" + "|".join(calendar.month_name[1:]) + "), \d{3,}"
    return existence_check(text, [regex], err, msg)


@memoize
def check_month_of_year(text):
    """Check the text."""
    err = "dates_times.dates"
    msg = u"When specifying a month and year, 'of' is unnecessary."

    regex = "(?:" + "|".join(calendar.month_name[1:]) + ") of \d{3,}"
    return existence_check(text, [regex], err, msg)



# -*- coding: utf-8 -*-

"""-er vs. -or."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check_er_or(text):
    """-er vs. -or."""
    err = "spelling.er_or"
    msg = "-er vs. -or. '{}' is the preferred spelling."

    preferences = [

        ["abductor",            ["abducter"]],
        ["abettor",             ["abbeter"]],
        ["acquirer",            ["acquiror"]],
        ["adapter",             ["adaptor"]],
        ["collector",           ["collecter"]],
        ["conjurer",            ["conjuror"]],
        ["corrupter",           ["corruptor"]],
        ["digester",            ["digestor"]],
        ["dispenser",           ["dispensor"]],
        ["distributor",         ["distributer"]],
        ["endorser",            ["endorsor"]],
        ["eraser",              ["erasor"]],
        ["idolater",            ["idolator"]],
        ["impostor",            ["imposter"]],
        ["infiltrator",         ["infiltrater"]],
        ["investor",            ["invester"]],
        ["manipulator",         ["manipulater"]],
        ["mortgagor",           ["mortgager"]],
        ["persecutor",          ["persecuter"]],
        ["promoter",            ["promotor"]],
        ["promoter",            ["promotor"]],
        ["purveyor",            ["purveyer"]],
        ["requester",           ["requestor"]],
        ["reviser",             ["revisor"]],
        ["surveyor",            ["surveyer"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



# -*- coding: utf-8 -*-
u"""a.m. / p.m.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      a.m. & p.m.
date:       2014-06-10 12:31:19
categories: writing
---

"""
from proselint.tools import memoize, existence_check


@memoize
def check_lowercase_periods(text):
    """Check the text."""
    err = "dates_times.am_pm.lowercase_periods"
    msg = u"With lowercase letters, the periods are standard."

    return existence_check(text, ["\d{1,2} ?[ap]m"], err, msg)


@memoize
def check_spacing(text):
    """Check the text."""
    err = "dates_times.am_pm.spacing"
    msg = u"It's standard to put a space before 'a.m.' or 'p.m.'."

    return existence_check(text, ["\d{1,2}[ap]\.?m\.?"], err, msg)


@memoize
def check_midnight_noon(text):
    """Check the text."""
    err = "dates_times.am_pm.midnight_noon"
    msg = (u"12 a.m. and 12 p.m. are wrong and confusing."
           " Use 'midnight' or 'noon'.")

    return existence_check(text, ["12 ?[ap]\.?m\.?"], err, msg)


@memoize
def check_redundancy(text):
    """Check the text."""
    err = "dates_times.am_pm.midnight_noon"
    msg = (u"'a.m.' is always morning; 'p.m.' is always night.")

    list = [
        "\d{1,2} ?a\.?m\.? in the morning",
        "\d{1,2} ?p\.?m\.? in the evening",
        "\d{1,2} ?p\.?m\.? at night",
        "\d{1,2} ?p\.?m\.? in the afternoon",
    ]

    return existence_check(text, list, err, msg, join=True)



# -*- coding: utf-8 -*-

"""-able vs. -atable."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check_able_atable(text):
    """-able vs. -ible."""
    err = "spelling.able_atable"
    msg = "-able vs. -atable. '{}' is the preferred spelling."

    preferences = [

        ["abbreviable",       ["abbreviatable"]],
        ["abdicable",         ["abdicatable"]],
        ["abrogable",         ["abrogatable"]],
        ["accommodable",      ["accommodatable"]],
        ["accumulable",       ["accumulatable"]],
        ["activable",         ["activatable"]],
        ["administrable",     ["administratable"]],
        ["adulterable",       ["adulteratable"]],
        ["affiliable",        ["affiliatable"]],
        ["aggregable",        ["aggregatable"]],
        ["agitable",          ["agitatable"]],
        ["alienable",         ["alienatable"]],
        ["alleviable",        ["alleviatable"]],
        ["allocable",         ["allocatable"]],
        ["ameliorable",       ["amelioratable"]],
        ["annihilable",       ["annihilatable"]],
        ["appreciable",       ["appreciatable"]],
        ["appropriable",      ["appropriatable"]],
        ["arbitrable",        ["arbitratable"]],
        ["articulable",       ["articulatable"]],
        ["calculable",        ["calculatable"]],
        ["communicable",      ["communicatable"]],
        ["compensable",       ["compensatable"]],
        ["confiscable",       ["confiscatable"]],
        ["corroborable",      ["corroboratable"]],
        ["cultivable",        ["cultivatable"]],
        ["delegable",         ["delegatable"]],
        ["delineable",        ["delineatable"]],
        ["demonstrable",      ["demonstratable"]],
        ["detonable",         ["detonatable"]],
        ["differentiable",    ["differentiatable"]],
        ["eradicable",        ["eradicatable"]],
        ["evacuable",         ["evacuatable"]],
        ["evaluable",         ["evaluatable"]],
        ["expropriable",      ["expropriatable"]],
        ["generable",         ["generatable"]],
        ["indicable",         ["indicatable"]],
        ["inebriable",        ["inebriatable"]],
        ["inextirpable",      ["inextirpatable"]],
        ["inextricable",      ["inextricatable"]],
        ["infatuable",        ["infatuatable"]],
        ["infuriable",        ["infuriatable"]],
        ["invalidable",       ["invalidatable"]],
        ["investigable",      ["investigatable"]],
        ["isolable",          ["isolatable"]],
        ["litigable",         ["litigatable"]],
        ["manipulable",       ["manipulatable"]],
        ["medicable",         ["medicatable"]],
        ["navigable",         ["navigatable"]],
        ["obligable",         ["obligatable"]],
        ["obviable",          ["obviatable"]],
        ["operable",          ["operatable"]],
        ["originable",        ["originatable"]],
        ["participable",      ["participatable"]],
        ["penetrable",        ["penetratable"]],
        ["perpetrable",       ["perpetratable"]],
        ["perpetuable",       ["perpetuatable"]],
        ["predicable",        ["predicatable"]],
        ["propagable",        ["propagatable"]],
        ["regulable",         ["regulatable"]],
        ["replicable",        ["replicatable"]],
        ["repudiable",        ["repudiatable"]],
        ["segregable",        ["segregatable"]],
        ["separable",         ["separatable"]],
        ["subjugable",        ["subjugatable"]],
        ["vindicable",        ["vindicatable"]],
        ["violable",          ["violatable"]],
        ["vitiable",          ["vitiatable"]]
    ]

    return preferred_forms_check(text, preferences, err, msg)



"""Test garner.dates."""
from __future__ import absolute_import
from __future__ import print_function

from .check import Check
from proselint.checks.dates_times import dates


class TestCheck(Check):
    """Test class for garner.dates."""

    __test__ = True

    def test_50s_hyphenation(self):
        """Find uneeded hyphen in 50's."""
        text = """The 50's were swell."""
        errors = dates.check_decade_apostrophes_short(text)
        assert len(errors) == 1

    def test_50_Cent_hyphenation(self):
        """Don't flag 50's when it refers to 50 Cent's manager."""
        text = """
            Dr. Dre suggested to 50's manager that he look into signing
            Eminem to the G-Unit record label.
        """
        errors = dates.check_decade_apostrophes_short(text)
        assert len(errors) == 0

    def test_dash_and_from(self):
        """Test garner.check_dash_and_from."""
        text = """From 1999-2002, Sally served as chair of the committee."""
        errors = dates.check_dash_and_from(text)
        print(errors)
        assert len(errors) == 1



"""Test dfw.uncomparables."""
from __future__ import absolute_import

from .check import Check
from proselint.checks.uncomparables import misc as chk


class TestCheck(Check):
    """The test class for dfw.uncomparables."""

    __test__ = True

    @property
    def this_check(self):
        """Bolierplate."""
        return chk

    def test_sample_phrases(self):
        """Find 'very unique'."""
        assert not self.passes("""This sentence is very unique.""")

    def test_linebreaks(self):
        """Handle linebreaks correctly."""
        assert not self.passes("""This sentence is very\nunique.""")

    def test_constitutional(self):
        """Don't flag 'more perfect'."""
        assert self.passes("""A more perfect union.""")



"""Waxed lyrical.

---
layout:     post
source:     Fowler's Modern English Usage
source_url: bit.ly/1YBG8QJ
title:      Waxed lyrical
date:       2016-03-10 14:48:42
categories: writing
---

Fowler's says:
Its primary meaning 'grow larger, increase' (as opposed to 'wane') leads
naturally to the sense 'pass into a specified state or mood, begin to use a
specified tone. In this meaning a following modifier must be an adj. not an
adverb ('He waxed enthusiastic [not enthusiastically] about Australia').
"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "misc.waxed"
    msg = u"The modifier following 'waxed' must be an adj.: '{}' is correct"

    waxes = ["wax", "waxes", "waxed", "waxing"]
    modifiers = [("ebullient", "ebulliently"),
                 ("ecstatic", "ecstatically"),
                 ("eloquent", "eloquently"),
                 ("enthusiastic", "enthusiastically"),
                 ("euphoric", "euphorically"),
                 ("indignant", "indignantly"),
                 ("lyrical", "lyrically"),
                 ("melancholic", "melancholically"),
                 ("metaphorical", "metaphorically"),
                 ("nostalgic", "nostalgically"),
                 ("patriotic", "patriotically"),
                 ("philosophical", "philosophically"),
                 ("poetic", "poetically"),
                 ("rhapsodic", "rhapsodically"),
                 ("romantic", "romantically"),
                 ("sentimental", "sentimentally")
                 ]

    def pairs(word):
        return [[word + ' ' + pair[0], [word + ' ' + pair[1]]]
                for pair in modifiers]

    preferred = []
    for word in waxes:
        preferred += pairs(word)

    return preferred_forms_check(text, preferred, err, msg)



"""Unit tests for strunk_white_eos."""
# from __future__ import absolute_import


# from .check import Check
# from proselint.checks.strunkwhite import elementary_composition as chk


# class TestCheck(Check):
#     """Define the suite of checks."""

#     __test__ = True

#     @property
#     def this_check(self):
#         return chk

#     def test_with_utilized(self):
#         """Don't produce an error when 'use' is used correctly."""
#         assert self.check("I use a hammer to drive nails into wood.")

#     def test_no_utilized(self):
#         """Produce an error when 'utilize' is used in place of 'use'."""
#         assert not self.check("I utilize a hammer to drive nails into wood.")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""General-purpose tools shared across linting checks."""
from __future__ import print_function
from __future__ import unicode_literals
import sys
import traceback
import os
import shelve
import dbm
import inspect
import functools
import re
import hashlib

_cache_shelves = dict()


def close_cache_shelves():
    """Close previously opened cache shelves."""
    for pth, cache in _cache_shelves.items():
        cache.close()
    _cache_shelves.clear()


def close_cache_shelves_after(f):
    """Decorator that insures cache shelves are closed after the call."""
    @functools.wraps(f)
    def wrapped(*args, **kwargs):
        f(*args, **kwargs)
        close_cache_shelves()
    return wrapped


def _get_cache(cachepath):
    if cachepath in _cache_shelves:
        return _cache_shelves[cachepath]

    try:
        cache = shelve.open(cachepath, protocol=2)
    except dbm.error:
        # dbm error on open - delete and retry
        print('Error (%s) opening %s - will attempt to delete and re-open.' %
              (sys.exc_info()[1], cachepath))
        try:
            os.remove(cachepath)
            cache = shelve.open(cachepath, protocol=2)
        except Exception:
            print('Error on re-open: %s' % sys.exc_info()[1])
            cache = None
    except Exception:
        # unknown error
        print('Could not open cache file %s, maybe name collision. '
              'Error: %s' % (cachepath, traceback.format_exc()))
        cache = None

    # Don't fail on bad caches
    if cache is None:
        print('Using in-memory shelf for cache file %s' % cachepath)
        cache = shelve.Shelf(dict())

    _cache_shelves[cachepath] = cache
    return cache


def memoize(f):
    """Cache results of computations on disk."""
    # Determine the location of the cache.
    cache_dirname = os.path.join(os.path.expanduser("~"), ".proselint")

    # Create the cache if it does not already exist.
    if not os.path.isdir(cache_dirname):
        os.mkdir(cache_dirname)

    cache_filename = f.__module__ + "." + f.__name__
    cachepath = os.path.join(cache_dirname, cache_filename)

    @functools.wraps(f)
    def wrapped(*args, **kwargs):

        # handle instance methods
        if hasattr(f, '__self__'):
            args = args[1:]

        signature = (f.__module__ + '.' + f.__name__).encode("utf-8")

        tempargdict = inspect.getcallargs(f, *args, **kwargs)

        for item in list(tempargdict.items()):
            signature += item[1].encode("utf-8")

        key = hashlib.sha256(signature).hexdigest()

        try:
            cache = _get_cache(cachepath)
            return cache[key]
        except KeyError:
            value = f(*args, **kwargs)
            cache[key] = value
            cache.sync()
            return value
        except TypeError:
            call_to = f.__module__ + '.' + f.__name__
            print('Warning: could not disk cache call to %s;'
                  'it probably has unhashable args. Error: %s' %
                  (call_to, traceback.format_exc()))
            return f(*args, **kwargs)

    return wrapped


def reverse(text):
    """Reverse a string. This is here as a demo of memoization."""
    return text[::-1]


def line_and_column(text, position):
    """Return the line number and column of a position in a string."""
    position_counter = 0
    for idx_line, line in enumerate(text.splitlines(True)):
        if (position_counter + len(line.rstrip())) > position:
            return (idx_line, position - position_counter)
        else:
            position_counter += len(line)


def consistency_check(text, word_pairs, err, msg, offset=0):
    """Build a consistency checker for the given word_pairs."""
    errors = []

    msg = " ".join(msg.split())

    for w in word_pairs:
        match1 = [m for m in re.finditer(w[0], text)]
        match2 = [m for m in re.finditer(w[1], text)]

        if len(match1) > 0 and len(match2) > 0:

            if len(match1) > len(match2):
                for m in match2:
                    errors.append((
                        m.start() + offset,
                        m.end() + offset,
                        err,
                        msg.format(m.group(0), w[0])))
            else:
                for m in match1:
                    errors.append((
                        m.start() + offset,
                        m.end() + offset,
                        err,
                        msg.format(m.group(0), w[1])))

    return errors


def preferred_forms_check(text, list, err, msg, ignore_case=True, offset=0,
                          max_errors=float("inf")):
    """Build a checker that suggests the preferred form."""
    if ignore_case:
        flags = re.IGNORECASE
    else:
        flags = 0

    msg = " ".join(msg.split())

    errors = []
    regex = u"[\W^]{}[\W$]"
    for p in list:
        for r in p[1]:
            for m in re.finditer(regex.format(r), text, flags=flags):
                txt = m.group(0).strip()
                errors.append((
                    m.start() + 1 + offset,
                    m.end() + offset,
                    err,
                    msg.format(p[0], txt)))

    errors = truncate_to_max(errors, max_errors)

    return errors


def existence_check(text, list, err, msg, ignore_case=True,
                    str=False, max_errors=float("inf"), offset=0,
                    require_padding=True, dotall=False,
                    excluded_topics=None, join=False):
    """Build a checker that blacklists certain words."""
    flags = 0

    msg = " ".join(msg.split())

    if ignore_case:
        flags = flags | re.IGNORECASE

    if str:
        flags = flags | re.UNICODE

    if dotall:
        flags = flags | re.DOTALL

    if require_padding:
        regex = u"(?:^|\W){}[\W$]"
    else:
        regex = u"{}"

    errors = []

    # If the topic of the text is in the excluded list, return immediately.
    if excluded_topics:
        tps = topics(text)
        if any([t in excluded_topics for t in tps]):
            return errors

    rx = "|".join(regex.format(w) for w in list)
    for m in re.finditer(rx, text, flags=flags):
        txt = m.group(0).strip()
        errors.append((
            m.start() + 1 + offset,
            m.end() + offset,
            err,
            msg.format(txt)))

    errors = truncate_to_max(errors, max_errors)

    return errors


def truncate_to_max(errors, max_errors):
    """If max_errors was specified, truncate the list of errors.

    Give the total number of times that the error was found elsewhere.
    """
    if len(errors) > max_errors:
        start1, end1, err1, msg1 = errors[0]

        if len(errors) == (max_errors + 1):
            msg1 += " Found once elsewhere."
        else:
            msg1 += " Found {} times elsewhere.".format(len(errors))

        errors = errors[1:max_errors]
        errors = [(start1, end1, err1, msg1)] + errors

    return errors


def is_quoted(position, text):
    """Determine if the position in the text falls within a quote."""
    def matching(quotemark1, quotemark2):
        straight = '\"\''
        curly = '“”'
        if quotemark1 in straight and quotemark2 in straight:
            return True
        if quotemark1 in curly and quotemark2 in curly:
            return True
        else:
            return False

    def find_ranges(text):
        s = 0
        q = pc = ''
        start = None
        ranges = []
        seps = " .,:;-\r\n"
        quotes = ['\"', '“', '”', "'"]
        for i, c in enumerate(text + "\n"):
            if s == 0 and c in quotes and pc in seps:
                start = i
                s = 1
                q = c
            elif s == 1 and matching(c, q):
                s = 2
            elif s == 2:
                if c in seps:
                    ranges.append((start+1, i-1))
                    start = None
                    s = 0
                else:
                    s = 1
            pc = c
        return ranges

    def position_in_ranges(ranges, position):
        for start, end in ranges:
            if start <= position < end:
                return True
        return False

    return position_in_ranges(find_ranges(text), position)


def detector_50_Cent(text):
    """Determine whether 50 Cent is a topic."""
    keywords = [
        "50 Cent",
        "rap",
        "hip hop",
        "Curtis James Jackson III",
        "Curtis Jackson",
        "Eminem",
        "Dre",
        "Get Rich or Die Tryin'",
        "G-Unit",
        "Street King Immortal",
        "In da Club",
        "Interscope",
    ]
    num_keywords = sum(word in text for word in keywords)
    return ("50 Cent", float(num_keywords > 2))


def topics(text):
    """Return a list of topics."""
    detectors = [
        detector_50_Cent
    ]
    ts = []
    for detector in detectors:
        ts.append(detector(text))

    return [t[0] for t in ts if t[1] > 0.95]


def context(text, position, level="paragraph"):
    """Get sentence or paragraph that surrounds the given position."""
    if level == "sentence":
        pass
    elif level == "paragraph":
        pass

    return ""



# -*- coding: utf-8 -*-
"""Checks that links are viable.

---
layout:     post
source:     SublimeLinter-annotations
source_url: http://bit.ly/16Q7H41
title:      broken links
date:       2014-06-10 12:31:19
categories: writing
---

Check that links are not not broken.

"""
from proselint.tools import memoize
from future import standard_library
import re
try:
    import urllib.request as urllib_request  # for Python 3
except ImportError:
    import urllib2 as urllib_request         # for Python 2
from socket import error as SocketError
standard_library.install_aliases()


@memoize
def check(text):
    """Check the text."""
    err = "links.valid"
    msg = u"Broken link: {}"

    regex = re.compile(
        r"""(?i)\b((?:https?://|www\d{0,3}[.]
        |[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+
        |(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)
        |[^\s`!()\[\]{};:\'".,<>?\xab\xbb\u201c\u201d\u2018\u2019\u21a9]))""",
        re.U)

    errors = []
    for m in re.finditer(regex, text):
        url = m.group(0).strip()

        if "http://" not in url and "https://" not in url:
            url = "http://" + url

        if is_broken_link(url):
            errors.append((m.start(), m.end(), err, msg.format(url)))

    return errors


@memoize
def is_broken_link(url):
    """Determine whether the link returns a 404 error."""
    try:
        request = urllib_request.Request(
            url, headers={'User-Agent': 'Mozilla/5.0'})
        urllib_request.urlopen(request).read()
        return False
    except urllib_request.URLError:
        return True
    except SocketError:
        return True



# -*- coding: utf-8 -*-
"""Sexism.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      sexism
date:       2014-06-10 12:31:19
categories: writing
---

Points out sexist language.

"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "sexism.misc"
    msg = "Gender bias. Use '{}' instead of '{}'."

    sexism = [
        ["anchor",           ["anchorman", "anchorwoman", "anchorperson"]],
        ["chair",            ["chairman", "chairwoman", "chairperson"]],
        ["drafter",          ["draftman", "draftwoman", "draftperson"]],
        ["ombuds",           ["ombudsman", "ombudswoman", "ombudsperson"]],
        ["tribe member",     ["tribesman", "tribeswoman", "tribesperson"]],
        ["police officer",   ["policeman", "policewoman", "policeperson"]],
        ["firefighter",      ["fireman", "firewoman", "fireperson"]],
        ["mail carrier",     ["mailman", "mailwoman", "mailperson"]],
        ["history",          ["herstory"]],
        ["women",            ["womyn"]],
        ["poet",             ["poetess"]],
        ["author",           ["authoress"]],
        ["waiter",           ["waitress"]],
        ["lawyer",           ["lady lawyer"]],
        ["doctor",           ["woman doctor"]],
        ["bookseller",       ["female booksalesman"]],
        ["air pilot",        ["femaile airman"]],
        ["executor",         ["executrix"]],
        ["prosecutor",       ["prosecutrix"]],
        ["testator",         ["testatrix"]],
        ["husband and wife", ["man and wife"]],
        ["chairs",           ["chairmen and chairs"]],
        ["men and women",    ["men and girls"]],
        ["comedian",         ["comedienne"]],
        ["confidant",        ["confidante"]],
        ["scientist",        ["woman scientist"]],
        ["scientists",       ["women scientists"]]
        # ["hero",             ["heroine"]]
    ]

    return preferred_forms_check(text, sexism, err, msg, ignore_case=False)



# -*- coding: utf-8 -*-
"""Mixed one vs. two spaces after a period.

---
layout:     post
source:     Consistency.
source_url: ???
title:      Mixed use of 1 vs. 2 spaces after a period.
date:       2014-06-10 12:31:19
categories: writing
---

Points out instances where there are two conventions, 1 vs. 2 spaces after
a period, in the same document.

"""
from proselint.tools import consistency_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "consistency.spacing"
    msg = "Inconsistent spacing after period (1 vs. 2 spaces)."

    regex = ["[\.\?!] [A-Z]", "[\.\?!]  [A-Z]"]
    return consistency_check(text, [regex], err, msg)



# -*- coding: utf-8 -*-
u"""Comparing uncomparables.

---
layout:     post
source:     David Foster Wallace
source_url: http://www.telegraph.co.uk/a/9715551
title:      Comparing an uncomparable
date:       2014-06-10 12:31:19
categories: writing
---

David Foster Wallace says:

> This is one of a class of adjectives, sometimes called “uncomparables”, that
can be a little tricky. Among other uncomparables are precise, exact, correct,
entire, accurate, preferable, inevitable, possible, false; there are probably
two dozen in all. These adjectives all describe absolute, non-negotiable
states: something is either false or it’s not; something is either inevitable
or it’s not. Many writers get careless and try to modify uncomparables with
comparatives like more and less or intensives like very. But if you really
think about them, the core assertions in sentences like “War is becoming
increasingly inevitable as Middle East tensions rise”; “Their cost estimate was
more accurate than the other firms’”; and “As a mortician, he has a very unique
attitude” are nonsense. If something is inevitable, it is bound to happen; it
cannot be bound to happen and then somehow even more bound to happen. Unique
already means one-of-a-kind, so the adj. phrase very unique is at best
redundant and at worst stupid, like “audible to the ear” or “rectangular in
shape”. You can blame the culture of marketing for some of this difficulty.
As the number and rhetorical volume of US ads increase, we become inured to
hyperbolic language, which then forces marketers to load superlatives and
uncomparables with high-octane modifiers (special - very special -
Super-special! - Mega-Special!!), and so on. A deeper issue implicit in the
problem of uncomparables is the dissimilarities between Standard Written
English and the language of advertising. Advertising English, which probably
deserves to be studied as its own dialect, operates under different syntactic
rules than SWE, mainly because AE’s goals and assumptions are different.
Sentences like “We offer a totally unique dining experience”; “Come on down and
receive your free gift”; and “Save up to 50 per cent… and more!” are perfectly
OK in Advertising English — but this is because Advertising English is aimed at
people who are not paying close attention. If your audience is by definition
involuntary, distracted and numbed, then free gift and totally unique stand a
better chance of penetrating — and simple penetration is what AE is all about.
One axiom of Standard Written English is that your reader is paying close
attention and expects you to have done the same.
"""
import re
from proselint.tools import memoize
import itertools


@memoize
def check(text):
    """Check the text."""
    err = "uncomparables.misc"
    msg = "Comparison of an uncomparable: '{}' is not comparable."

    comparators = [
        "most",
        "more",
        "less",
        "least",
        "very",
        "quite",
        "largely",
        "extremely",
        "increasingly"
    ]

    uncomparables = [
        "absolute",
        "adequate",
        "chief",
        "complete",
        "correct",
        "devoid",
        "entire",
        "false",
        "fatal",
        "favorite",
        "final",
        "ideal",
        "impossible",
        "inevitable",
        "infinite",
        "irrevocable",
        "main",
        "manifest",
        "only",
        "paramount",
        "perfect",
        "perpetual",
        "possible",
        "preferable",
        "principal",
        "singular",
        "stationary",
        "sufficient",
        "true",
        "unanimous",
        "unavoidable",
        "unbroken",
        "uniform",
        "unique",
        "universal",
        "void",
        "whole",
    ]

    exceptions = [
        ("more", "perfect"),
        ("more", "possible")  # FIXME
    ]

    all = ["\\b" + i[0] + "\s" + i[1] + "[\W$]" for i in itertools.product(
           comparators, uncomparables) if i not in exceptions]

    occ = re.finditer("|".join(all), text.lower())
    return [(o.start(), o.end(), err, msg.format(o.group(0))) for o in occ]



# -*- coding: utf-8 -*-
"""Metaconcepts.

---
layout:     post
source:     Pinker's book on writing
source_url: ???
title:      misuse of scare quotes
date:       2014-06-10 12:31:19
categories: writing
---

Point out misuse of scare quotes.

"""
# from proselint.tools import memoize, existence_check


# @memoize
# def check(text):
#     """Suggest the preferred forms."""
#     err = "pinker.metaconcepts"
#     msg = "Misuse of 'scare quotes'. Delete them."

#     narcisissm = [
#         "the 'take-home message'",
#     ]

#     return existence_check(text, narcisissm, err, msg)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Compute the lintscore on the corpus."""

from __future__ import print_function
from builtins import input
from builtins import int

import os
import subprocess
import re

proselint_path = os.path.dirname(os.path.realpath(__file__))


def score(check=None):
    """Compute the linter's score on the corpus.

    Proselint's score reflects the desire to have a linter that catches many
    errors, but which takes false alarms seriously. It is better not to say
    something than to say the wrong thing, and the harm from saying the wrong
    thing is greater than the benefit of saying the right thing. Thus our score
    metric is defined as:

        TP * (TP / (FP + TP)) ^ k,

    where TP is the number of true positives (hits), FP is the number of
    false positives (false alarms), and k > 0 is a temperature parameter that
    determines the penalty for imprecision. In general, we should choose a
    large value of k, one that strongly discourages the creation of rules that
    can't be trusted. Suppose that k = 2. Then if the linter detects 100
    errors, of which 10 are false positives, the score is 81.
    """
    tp = 0
    fp = 0

    parent_directory = os.path.dirname(proselint_path)
    path_to_corpus = os.path.join(parent_directory, "corpora", "0.1.0")
    for root, _, files in os.walk(path_to_corpus):
        files = [f for f in files if f.endswith(".md")]
        for f in files:
            fullpath = os.path.join(root, f)

            # Run the linter.
            print("Linting {}".format(f))
            out = subprocess.check_output(
                "proselint {}".format(fullpath), shell=True)

            # Determine the number of errors.
            regex = r".+?:(?P<line>\d+):(?P<col>\d+): (?P<message>.+)"
            num_errors = len(tuple(re.finditer(regex, out)))
            print("Found {} errors.".format(num_errors))

            # Open the document.
            subprocess.call("{} {}".format("open", fullpath), shell=True)

            # Ask the scorer how many of the errors were false alarms?
            input_val = None
            while not isinstance(input_val, int):
                try:
                    input_val = input("# of false alarms? ")
                    if input_val == "exit":
                        return
                    else:
                        input_val = int(input_val)
                        fp += input_val
                        tp += (num_errors - input_val)
                except ValueError:
                    pass

            print("Currently {} hits and {} false alarms\n---".format(tp, fp))

    if (tp + fp) > 0:
        return tp * (1.0 * tp / (tp + fp)) ** 2
    else:
        return 0



# -*- coding: utf-8 -*-
"""Excessive apologizing.

---
layout:     post
source:     Pinker's book on writing
source_url: ???
title:      excessive apologizing
date:       2014-06-10 12:31:19
categories: writing
---

Points out excessive apologizing.

"""
from proselint.tools import memoize, existence_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "pinker.apologizing"
    msg = "Excessive apologizing."

    narcisissm = [
        "More research is needed",
    ]

    return existence_check(text, narcisissm, err, msg)



# -*- coding: utf-8 -*-
"""On 'the N word'.

---
layout:     post
source:     Louis CK
source_url: https://youtu.be/dF1NUposXVQ?t=30s
title:      the 'n-word'
date:       2014-06-10 12:31:19
categories: writing
---

Take responsibility with the shitty words you wanna say.

"""
from proselint.tools import existence_check, memoize


@memoize
def check_repeated_exclamations(text):
    """Check the text."""
    err = "cursing.nword"
    msg = u"Take responsibility for the shitty words you want to say."

    list = [
        "the n-word",
    ]

    return existence_check(text, list, err, msg)



"""Weasel words.

---
layout:     post
source:     write-good
source_url: https://github.com/btford/write-good
title:      Weasel words.
date:       2014-06-10 12:31:19
categories: writing
---

Weasel words clearly weaken various aspects of a number of your sentences.

"""


# def check(text):

#     error_code = "weasel_words.misc"
#     msg = "Weasel words present."

#     return [(1, 1, error_code, msg)]



# -*- coding: utf-8 -*-

"""Redundancy."""

from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "redundancy.wallace"
    msg = "Redundancy. Use '{}' instead of '{}'."

    redundancies = [
        ["rectangular",        ["rectangular in shape"]],
        ["audible",            ["audible to the ear"]],
    ]
    return preferred_forms_check(text, redundancies, err, msg)


@memoize
def check_garner(text):
    """Suggest the preferred forms.

    source:     Garner's Modern American Usage
    source_url: http://bit.ly/1T4alrY
    """
    err = "redundancy.garner"
    msg = "Redundancy. Use '{}' instead of '{}'."

    redundancies = [
        ["adequate",          ["adequate enough"]],
        ["admitted",          ["self-admitted"]],
        ["affidavit",         ["sworn affidavit"]],
        ["agreement",         ["mutual agreement"]],
        ["alumnus",           ["former alumnus"]],
        ["antithetical",      ["directly antithetical"]],
        ["approximately",     ["approximately about"]],
        ["associate",         ["associate together(?: in groups)?"]],
        ["bivouac",           ["temporary bivouac", "bivouac camp"]],
        ["blend",             ["blend together"]],
        ["but",               ["but nevertheless"]],
        ["charged with...",   ["accused of a charge"]],
        ["circumstances of",  ["circumstances surrounding"]],
        ["circumstances",     ["surrounding circumstances"]],
        ["close",             ["close proximity"]],
        ["collaborate",       ["collaborate together"]],
        ["collaborator",      ["fellow collaborator"]],
        ["collaborators",     ["fellow collaborators"]],
        ["collocated",        ["collocated together"]],
        ["colleagues",        ["fellow colleagues"]],
        ["combine",           ["combine together"]],
        ["complacent",        ["self-complacent"]],
        ["confessed",         ["self-confessed"]],
        ["connect",           ["connect together"]],
        ["consensus",         ["(?:general )?consensus of opinion"]],
        ["consolidate",       ["consolidate together"]],
        ["continues to",      ["still continues to"]],
        ["contradictory",     ["mutually contradictory"]],
        ["cooperation",       ["mutual cooperation"]],
        ["couple",            ["couple together"]],
        ["crisis",            ["serious crisis"]],
        ["eliminate",         ["entirely eliminate"]],
        ["especially",        ["most especially"]],
        ["fact",              ["actual fact"]],
        ["facts",             ["true facts"]],
        ["forecast",          ["future forecast"]],
        ["founding fathers",  ["founding forefathers"]],
        ["free",              ["free and gratis"]],
        ["free",              ["free gratis"]],
        ["full",              ["completely full"]],
        ["fundamentals",      ["basic fundamentals"]],
        ["gift",              ["free gift"]],
        ["innovation",        ["new innovation"]],
        ["interact",          ["interact with each other"]],
        ["large",             ["large-size"]],
        ["meld",              ["meld together"]],
        ["merge",             ["merge together"]],
        ["mingle",            ["mingle together"]],
        ["mix",               ["mix together"]],
        ["mutual feelings",   ["mutual feelings for eachother"]],
        ["mutual respect",    ["mutual respect for each other"]],
        ["native citizen",    ["native-born citizen"]],
        ["necessity",         ["absolute necessity"]],
        ["obvious",           ["blatantly obvious"]],
        ["pause",             ["pause for a moment"]],
        ["planning",          ["advance planning"]],
        ["plans",             ["future plans"]],
        ["pooled",            ["pooled together"]],
        ["potable water",     ["potable drinking water"]],
        ["potable water",     ["potable drinking water"]],
        ["recruit",           ["new recruit"]],
        ["reelected",         ["reelected for another term"]],
        ["refer",             ["refer back"]],
        ["regress",           ["regress back"]],
        ["repay them",        ["repay them back"]],
        ["repay",             ["repay back"]],
        ["repeat",            ["repeat again"]],
        ["repeat",            ["repeat back"]],
        ["repeat",            ["repeat the same"]],
        ["repeated",          ["repeated the same"]],
        ["reprieve",          ["temporary reprieve"]],
        ["respite",           ["brief respite"]],
        ["retirement",        ["retiral", "retiracy"]],
        ["retreat",           ["retreat back"]],
        ["return",            ["return back"]],
        ["scrutinize",        ["closely scrutinize"]],
        ["software",          ["software program"]],
        ["surrounded",        ["surrounded on all sides"]],
        ["the nation",        ["the whole entire nation"]],
        ["throughout the",    ["throughout the entire"]],
        ["timpani",           ["timpani drum"]],
        ["twins",             ["pair of twins"]],
        ["vacancy",           ["unfilled vacancy"]],
        ["various",           ["various different"]],
        ["veteran",           ["former veteran"]],
        ["visible",           ["visible to the eye"]],
        ["vocation",          ["professional vocation"]],
        ["while",             ["while at the same time"]],
    ]

    return preferred_forms_check(text, redundancies, err, msg)


@memoize
def check_nordquist(text):
    """Suggest the preferred forms.

    source:     Richard Nordquist
    source_url: http://grammar.about.com/bio/Richard-Nordquist-22176.htm
    """
    err = "redundancy.nordquist"
    msg = "Redundancy. Use '{}' instead of '{}'."

    redundancies = [
        ["essential",          ["absolutely essential"]],
        ["necessary",          ["absolutely necessary"]],
        ["a.m.",               ["a.m. in the morning"]],
        ["p.m.",               ["p.m. at night"]],
    ]
    return preferred_forms_check(text, redundancies, err, msg)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Command line utility for proselint."""
from __future__ import print_function
from __future__ import absolute_import
from builtins import str


import click
import os
from .tools import (
    line_and_column,
    is_quoted,
    close_cache_shelves_after,
    close_cache_shelves
)
from . import checks as pl
import pkgutil
import subprocess
import re
import json
import importlib
import sys
from .score import score
from .version import __version__


CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
base_url = "proselint.com/"
proselint_path = os.path.dirname(os.path.realpath(__file__))
demo_file = os.path.join(proselint_path, "demo.md")


def run_initialization():
    """Run the initialization method for each check."""
    for loader, module_name, is_pkg in pkgutil.walk_packages(pl.__path__):
        module = loader.find_module(module_name).load_module(module_name)

        # Run the initialization.
        try:
            assert(hasattr(module.initialize, '__call__'))
            module.initialize()
        except Exception:
            pass


def lint(input_file, debug=False):
    """Run the linter on the input file."""
    # Load the options.
    options = json.load(open(os.path.join(proselint_path, '.proselintrc')))

    # Extract the checks.
    sys.path.append(proselint_path)
    checks = []
    check_names = [key for (key, val)
                   in list(options["checks"].items()) if val]

    for check_name in check_names:
        module = importlib.import_module("checks." + check_name)
        for d in dir(module):
            if re.match("check", d):
                checks.append(getattr(module, d))

    # Apply all the checks.
    text = input_file.read()
    errors = []
    for check in checks:
        if debug:
            click.echo(check.__module__ + "." + check.__name__)

        result = check(text)

        for error in result:
            (start, end, check, message) = error
            (line, column) = line_and_column(text, start)
            if not is_quoted(start, text):
                errors += [(check, message, line, column, start, end,
                           end - start, "warning", None)]

        if len(errors) > options["max_errors"]:
            break

    # Sort the errors by line and column number.
    errors = sorted(errors[:options["max_errors"]])

    return errors


def lintscore():
    """Compute the linter's score on a corpus."""
    return score()


def timing_test(corpus="0.1.0"):
    """Measure timing performance on the named corpus."""
    import time
    dirname = os.path.dirname
    corpus_path = os.path.join(
        dirname(dirname(os.path.realpath(__file__))), "corpora", corpus)
    start = time.time()
    for file in os.listdir(corpus_path):
        filepath = os.path.join(corpus_path, file)
        if ".md" == filepath[-3:]:
            subprocess.call(
                "proselint {} >/dev/null".format(filepath), shell=True)

    return time.time() - start


def clear_cache():
    """Delete the contents of the cache."""
    click.echo("Deleting the cache...")
    subprocess.call("find . -name '*.pyc' -delete", shell=True)
    subprocess.call(
        "rm -rfv proselint/cache > /dev/null && mkdir -p {}".format(
            os.path.join(os.path.expanduser("~"), ".proselint")),
        shell=True)


def errors_to_json(errors):
    """Convert the errors to JSON."""
    out = []
    for e in errors:
        out.append({
            "check": e[0],
            "message": e[1],
            "line": 1 + e[2],
            "column": 1 + e[3],
            "start": 1 + e[4],
            "end": 1 + e[5],
            "extent": e[6],
            "severity": e[7],
            "replacements": e[8],
        })

    return json.dumps(dict(status="success", data={"errors": out}))


def show_errors(filename, errors, output_json=False, compact=False):
    """Print the errors, resulting from lint, for filename."""
    if output_json:
        click.echo(errors_to_json(errors))

    else:
        for error in errors:

            (check, message, line, column, start, end,
             extent, severity, replacements) = error

            if compact:
                filename = "-"

            click.echo(
                filename + ":" +
                str(1 + line) + ":" +
                str(1 + column) + ": " +
                check + " " +
                message)


@click.command(context_settings=CONTEXT_SETTINGS)
@click.version_option(__version__, '--version', '-v', message='%(version)s')
@click.option('--initialize', '-i', is_flag=True)
@click.option('--debug', '-d', is_flag=True)
@click.option('--clean', '-c', is_flag=True)
@click.option('--score', '-s', is_flag=True)
@click.option('--json', '-j', 'output_json', is_flag=True)
@click.option('--time', '-t', is_flag=True)
@click.option('--demo', is_flag=True)
@click.option('--compact', is_flag=True)
@click.argument('paths', nargs=-1, type=click.Path())
@close_cache_shelves_after
def proselint(paths=None, version=None, initialize=None, clean=None,
              debug=None, score=None, output_json=None, time=None, demo=None,
              compact=None):
    """Define the linter command line API."""
    if time:
        click.echo(timing_test())
        return

    # Run the intialization.
    if initialize:
        run_initialization()
        return

    if score:
        click.echo(lintscore())
        return

    # In debug or clean mode, delete cache & *.pyc files before running.
    if debug or clean:
        clear_cache()

    # Use the demo file by default.
    if demo:
        paths = [demo_file]

    # Expand the list of directories and files.
    filepaths = extract_files(list(paths))

    # Lint the files
    num_errors = 0
    for fp in filepaths:
        try:
            f = click.open_file(fp, 'r+', encoding="utf-8")
            errors = lint(f, debug=debug)
            num_errors += len(errors)
            show_errors(fp, errors, output_json, compact=compact)
        except:
            pass

    # Return an exit code
    close_cache_shelves()
    if num_errors > 0:
        sys.exit(1)
    else:
        sys.exit(0)


def extract_files(files):
    """Expand list of paths to include all text files matching the pattern."""
    expanded_files = []
    legal_extensions = [".md", ".txt", ".rtf", ".html", ".tex", ".markdown"]

    for f in files:

        # If it's a directory, recursively walk through it and find the files.
        if os.path.isdir(f):
            for dir_, _, filenames in os.walk(f):
                for filename in filenames:
                    fn, file_extension = os.path.splitext(filename)
                    if file_extension in legal_extensions:
                        rel_dir = os.path.relpath(dir_, f)
                        rel_file = os.path.join(rel_dir, filename)
                        expanded_files.append(rel_file)

        # Otherwise add the file directly.
        else:
            expanded_files.append(f)

    return expanded_files


if __name__ == '__main__':
    proselint()



"""Lexical illusions.

---
layout:     post
source:     write-good
source_url: https://github.com/btford/write-good
title:      Lexical illusion present
date:       2014-06-10 12:31:19
categories: writing
---

A lexical illusion happens when a word word is unintentiall repeated twice, and
and this happens most often between line breaks.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "lexical_illusions.misc"
    msg = u"There's a lexical illusion here: a word is repeated."

    list = [
        "the\sthe",
        "am\sam",
        "has\shas"
    ]

    return existence_check(text, list, err, msg)



# -*- coding: utf-8 -*-
"""Not guilty beyond a reasonable doubt.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      Not guilty beyond a reasonable doubt.
date:       2016-03-09 15:50:31
categories: writing
---

This phrasing is ambiguous. The standard by which a jury decides criminal
charges is this: a defendant is guilty only if the evidence shows, beyond a
reasonable doubt, that he or she committed the crime. Otherwise, the defendant
is not guilty. Thus, we say that a defendant was not found "guilty beyond a
reasonable doubt."

If somebody is found not guilty, say "not guilty." Omit the standard
("beyond a reasonable doubt") to prevent a miscue.

Not guilty beyond a reasonable doubt
"""


from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "misc.not_guilty"
    msg = u"'not guilty beyond a reasonable doubt' is an ambiguous phrasing."
    regex = r"not guilty beyond (a |any )?reasonable doubt"

    return existence_check(text, [regex], err, msg)



# -*- coding: utf-8 -*-

"""Mixed metaphors."""

from proselint.tools import memoize, existence_check, preferred_forms_check


@memoize
def check_bottleneck(text):
    """Avoid mixing metaphors about bottles and their necks.

    source:     Sir Ernest Gowers
    source_url: http://bit.ly/1CQPH61
    """
    err = "mixed_metaphors.misc.bottleneck"
    msg = u"Mixed metaphor — bottles with big necks are easy to pass through."
    list = [
        "biggest bottleneck",
        "big bottleneck",
        "large bottleneck",
        "largest bottleneck",
        "world-wide bottleneck",
        "huge bottleneck",
        "massive bottleneck",
    ]

    return existence_check(text, list, err, msg, max_errors=1)


@memoize
def check_misc(text):
    """Avoid mixing metaphors.

    source:     Garner's Modern American Usage
    source_url: http://bit.ly/1T4alrY
    """
    err = "mixed_metaphors.misc.misc"
    msg = u"Mixed metaphor. Try '{}'."

    preferences = [

        ["cream rises to the top",    ["cream rises to the crop"]],
        ["fasten your seatbelts",     ["button your seatbelts"]],
        ["a minute to decompress",    ["a minute to decompose"]],
        ["sharpest tool in the shed", ["sharpest marble in the (shed|box)"]],
        ["not rocket science",        ["not rocket surgery"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



# -*- coding: utf-8 -*-
"""Currency.

---
layout:     post
source:     SublimeLinter-annotations
source_url: http://bit.ly/16Q7H41
title:      symbols
date:       2014-06-10 12:31:19
categories: writing
---

Symbols.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "misc.currency"
    msg = u"Incorrect use of symbols in {}."

    symbols = [
        "\$[\d]* ?(?:dollars|usd|us dollars)"
    ]

    return existence_check(text, symbols, err, msg)



# -*- coding: utf-8 -*-
"""Archaism.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      archaism
date:       2014-06-10 12:31:19
categories: writing
---

Archaism.

"""
from proselint.tools import existence_check, memoize


@memoize
def check(text):
    """Check the text."""
    err = "archaism.misc"
    msg = u"'{}' is archaic."

    archaisms = [
        "alack",
        "anent",
        # "anon",
        "begat",
        "belike",
        "betimes",
        "boughten",
        "brocage",
        "brokage",
        "camarade",
        "chiefer",
        "chiefest",
        "Christiana",
        "completely obsolescent",
        "cozen",
        "divers",
        "deflexion",
        "durst",
        "fain",
        "forsooth",
        "foreclose from",
        "haply",
        "howbeit",
        "illumine",
        "in sooth",
        "maugre",
        "meseems",
        "methinks",
        "nigh",
        "peradventure",
        "perchance",
        "saith",
        "shew",
        "sistren",
        "spake",
        "to wit",
        "verily",
        "whilom",
        "withal",
        "wot",
        "enclosed please find",
        "please find enclosed",
        "enclosed herewith",
        "enclosed herein",
        "inforce",
        "ex postfacto",
        "foreclose from",
        "forewent",
        "for ever",
        # "designer", when used to mean a plotter against Christ
        # "demean", when used to mean "to behave" in legal contexts
        # "by the bye", # variant, modern is "by the by"
        # "comptroller" # in british english
        # "abortive" Abortive is archaic in reference to abortions of fetuses,
        # except in the sense “causing an abortion.”
    ]

    return existence_check(text, archaisms, err, msg, join=True)



# -*- coding: utf-8 -*-
"""Psychological and psychiatric terms to avoid.

---
layout:     post
source:     Scott O. Lilienfeld, et al.
source_url: http://dx.doi.org/10.3389/fpsyg.2015.01100
title:      psychological and psychiatric terms to avoid
date:       2014-06-10 12:31:19
categories: writing
---

Psychological and psychiatric terms to avoid.

"""
from proselint.tools import preferred_forms_check, existence_check, memoize


@memoize
def check_lie_detector_test(text):
    """Suggest the preferred forms."""
    err = "psychology.lie_detector"
    msg = "Polygraph machines measure arousal, not lying per se. Try {}."

    list = [
        ["polygraph test",      ["lie detector test"]],
        ["polygraph machine",   ["lie detector machine"]],
    ]

    return preferred_forms_check(text, list, err, msg)


@memoize
def check_p_equals_zero(text):
    """Check for p = 0.000."""
    err = "psychology.p_equals_zero"
    msg = "Unless p really equals zero, you should use more decimal places."

    list = [
        "p = 0.00",
        "p = 0.000",
        "p = 0.0000",
    ]

    return existence_check(text, list, err, msg, join=True)


@memoize
def check_mental_telepathy(text):
    """Check for 'mental telepathy'."""
    err = "psychology.mental_telepathy"
    msg = "This is redundant because all purported telepathy is mental."

    return existence_check(text, ["mental telepathy"], err, msg)



"""Installation script for proselint."""

from setuptools import setup, find_packages
from proselint.version import __version__

base_url = 'http://github.com/amperser/proselint'

setup(
    name='proselint',
    version=__version__,
    description='A linter for prose',
    url=base_url,
    download_url="{}/tarball/{}".format(base_url, __version__),
    author='Amperser Labs',
    author_email='hello@amperser.com',
    license='BSD',
    packages=find_packages(),
    package_data={'': ['demo.md', '.proselintrc']},
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'proselint = proselint.command_line:proselint',
        ],
    },
    install_requires=[
        'click',
        'future',
        'six'
    ])



# -*- coding: utf-8 -*-
"""Skunked terms.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      Skunked terms.
date:       2014-06-10 12:31:19
categories: writing
---

Archaism.

"""
from proselint.tools import existence_check, memoize


@memoize
def check_skunked(text):
    """Check the text."""
    err = "skunked_terms.misc"
    msg = u"""'{}' is a bit of a skunked term — impossible to use without issue.
             Find some other way to say it."""

    skunked_terms = [
        "bona fides",
        "deceptively",
        "decimate",
        "effete",
        "fulsome",
        "hopefully",
        "impassionate",
        "Thankfully,",
    ]

    return existence_check(text, skunked_terms, err, msg)



# -*- coding: utf-8 -*-
"""Filthy words.

---
layout:     post
source:     George Carlin
source_url: https://youtu.be/kyBH5oNQOS0
title:      filthy words
date:       2014-06-10 12:31:19
categories: writing
---

Filthy words.

"""
from proselint.tools import existence_check, memoize


@memoize
def check_debased_language(text):
    """Check the text."""
    err = "cursing.filth"
    msg = u"""Nobody ever tells you this as a kid, but you're supposed to avoid
        this word."""

    list = [
        "shit",
        "piss",
        "fuck",
        "cunt",
        "cocksucker",
        "motherfucker",
        "tits",
        "fart",
        "turd",
        "twat",
    ]

    return existence_check(text, list, err, msg)



# -*- coding: utf-8 -*-
"""Needless variants.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      needless variants
date:       2014-06-10 12:31:19
categories: writing
---

Points out use of needless variants.

"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "needless_variants.misc"
    msg = "Needless variant. '{}' is the preferred form."

    preferences = [

        # Needless variants
        ["abolition",         ["abolishment"]],
        ["accessory",         ["accessary"]],
        ["accredit",          ["accreditate"]],
        ["accrual",           ["accruement"]],
        ["accumulate",        ["cumulate"]],
        ["accused",           ["accusee"]],
        ["acquaintance",      ["acquaintanceship"]],
        ["acquittal",         ["acquitment"]],
        ["administer",        ["administrate"]],
        ["administered",      ["administrated"]],
        ["administering",     ["administrating"]],
        ["adulterous",        ["adulterate"]],
        ["advisory",          ["advisatory"]],
        ["advocate",          ["advocator"]],
        ["alleger",           ["allegator"]],
        ["allusive",          ["allusory"]],
        ["ameliorate",        ["meliorate"]],
        ["amorous",           ["amative"]],
        ["amortization",      ["amortizement"]],
        ["amphibology",       ["amphiboly"]],
        ["anachronism",       ["parachronism"]],
        ["anecdotist",        ["anecdotalist"]],
        ["anilingus",         ["anilinctus"]],
        ["anticipatory",      ["anticipative"]],
        ["antithetical",      ["antithetic"]],
        ["applicable",        ["applicative"]],
        ["applicable",        ["applicatory"]],
        ["applicator",        ["applier"]],
        ["approbatory",       ["approbative"]],
        ["arbitrageur",       ["arbitrager"]],
        ["arsenious",         ["arsenous"]],
        ["ascendancy",        ["ascendance"]],
        ["ascendancy",        ["ascendence"]],
        ["ascendancy",        ["ascendency"]],
        ["authorial",         ["auctorial"]],
        ["averment",          ["averral"]],
        ["barbed wire",       ["barbwire"]],
        ["beneficent",        ["benefic"]],
        ["benign",            ["benignant"]],
        ["bestowal",          ["bestowment"]],
        ["betrothal",         ["betrothment"]],
        ["blameworthiness",   ["blamableness"]],
        ["buck naked",        ["butt naked"]],
        ["captor",            ["capturer"]],
        ["carte blanche",     ["carta blanca"]],
        ["casualties",        ["casualities"]],
        ["casualty",          ["casuality"]],
        ["catch fire",        ["catch on fire"]],
        ["catholically",      ["catholicly"]],
        ["ceasefire",         ["cease fire"]],
        ["cellphone",         ["cell phone", "cell-phone"]],
        ["channel",           ["channelize"]],
        ["chaplaincy",        ["chaplainship"]],
        ["chrysalis",         ["chrysalid"]],
        ["chrysalises",       ["chrysalids"]],
        ["cigarette",         ["cigaret"]],
        ["cliquish",          ["cliquey", "cliquy"]],
        ["cognitive",         ["cognitional"]],
        ["cohabit",           ["cohabitate"]],
        ["cohabitant",        ["cohabitor"]],
        ["collodion",         ["collodium"]],
        ["collusive",         ["collusory"]],
        ["commemorative",     ["commemoratory"]],
        ["commonage",         ["commonty"]],
        ["communicative",     ["communicatory"]],
        ["compensatory",      ["compensative"]],
        ["complacency",       ["complacence"]],
        ["complicit",         ["complicitous"]],
        ["compute",           ["computate"]],
        ["comrade",           ["camarade"]],
        ["conciliatory",      ["conciliative"]],
        ["concomitance",      ["concomitancy"]],
        ["condonation",       ["condonance"]],
        ["confirmatory",      ["confirmative"]],
        ["congruence",        ["congruency"]],
        ["connote",           ["connotate"]],
        ["consanguine",       ["consanguineal"]],
        ["conspicuousness",   ["conspicuity"]],
        ["conspirator",       ["conspiratorialist"]],
        ["constitutionalist", ["constitutionist"]],
        ["contemporaneous",   ["cotemporaneous"]],
        ["contemporary",      ["cotemporary"]],
        ["contigency",        ["contingence"]],
        ["contributory",      ["contributary"]],
        ["contumacy",         ["contumacity"]],
        ["convertible",       ["conversible"]],
        ["conveyance",        ["conveyal"]],
        ["corroborative",     ["corroboratory"]],
        ["coworker",          ["coemployee"]],
        ["curative",          ["curatory"]],
        ["daredevilry",       ["daredeviltry"]],
        ["deceptive",         ["deceptious"]],
        ["defamatory",        ["defamative"]],
        ["degenerative",      ["degeneratory"]],
        ["delimit",           ["delimitate"]],
        ["delusive",          ["delusory"]],
        ["denunciation",      ["denouncement"]],
        ["depositary",        ["depositee"]],
        ["depreciatory",      ["depreciative"]],
        ["deprivation",       ["deprival"]],
        ["derogatory",        ["derogative"]],
        ["destructible",      ["destroyable"]],
        ["dethrone",          ["disenthrone"]],
        ["detoxify",          ["detoxicate"]],
        ["detractive",        ["detractory"]],
        ["deuterogamy",       ["digamy"]],
        ["deviance",          ["deviancy"]],
        ["deviant",           ["deviationist"]],
        ["digitize",          ["digitalize"]],
        ["diminution",        ["diminishment"]],
        ["diplomat",          ["diplomatist"]],
        ["disciplinary",      ["disciplinatory"]],
        ["discriminating",    ["discriminant"]],
        ["disintegrative",    ["disintegratory"]],
        ["dismissal",         ["dismission"]],
        ["disorient",         ["disorientate"]],
        ["disoriented",       ["disorientated"]],
        ["disquiet",          ["disquieten"]],
        ["dissociate",        ["disassociate"]],
        ["distrait",          ["distraite"]],
        ["divergence",        ["divergency"]],
        ["divisible",         ["dividable"]],
        ["doctrinaire",       ["doctrinary"]],
        ["documentary",       ["documental"]],
        ["domesticate",       ["domesticize"]],
        ["doubt",             ["misdoubt"]],
        ["duplicative",       ["duplicatory"]],
        ["dutiful",           ["duteous"]],
        ["educationist",      ["educationalist"]],
        ["educative",         ["educatory"]],
        ["empanel",           ["impanel"]],
        ["encumbrance",       ["cumbrance"]],
        ["endow",             ["indow"]],
        ["endue",             ["indue"]],
        ["enigmas",           ["enigmatas"]],
        ["enlarge",           ["enlargen"]],
        ["epic",              ["epical"]],
        ["eroticism",         ["erotism"]],
        ["ethicist",          ["ethician"]],
        ["ex officio",        ["ex officiis"]],
        ["exculpatory",       ["exculpative"]],
        ["exigency",          ["exigence"]],
        ["exigent",           ["exigeant"]],
        ["exoticism",         ["exotism"]],
        ["expediency",        ["expedience"]],
        ["expedient",         ["expediential"]],
        ["expedient",         ["expediential"]],
        ["extendable",        ["extensible"]],
        ["eyeing",            ["eying"]],
        ["fief",              ["fiefdom"]],
        ["flagrancy",         ["flagrance"]],
        ["flatulence",        ["flatulency"]],
        ["fraudulent",        ["defraudulent"]],
        ["fraudulent",        ["fraudful"]],
        ["funereal",          ["funebrial"]],
        ["geographic",        ["geographical"]],
        ["geometric",         ["geometrical"]],
        ["goatherd",          ["goatherder"]],
        ["grievance",         ["aggrievance"]],
        ["gustatory",         ["gustatorial"]],
        ["habit",             ["habitude"]],
        ["henceforth",        ["henceforward"]],
        ["hesitancy",         ["hesitance"]],
        ["heterogeneous",     ["heterogenous"]],
        ["hierarchical",      ["hierarchic"]],
        ["hindmost",          ["hindermost"]],
        ["honoree",           ["honorand"]],
        ["hypostatize",       ["hypostasize"]],
        ["hysterical",        ["hysteric"]],
        ["idolize",           ["idolatrize"]],
        ["impersonation",     ["personation"]],
        ["impervious",        ["imperviable"]],
        ["importunity",       ["importunacy"]],
        ["impotence",         ["impotency"]],
        ["imprimatur",        ["imprimatura"]],
        ["improper",          ["improprietous"]],
        ["incitement",        ["incitation"]],
        ["inconsistency",     ["inconsistence"]],
        ["incriminate",       ["criminate"]],
        ["inculpatory",       ["culpatory"]],
        ["incurrence",        ["incurment"]],
        ["infrequent",        ["unfrequent"]],
        ["inhibitory",        ["inhibitive"]],
        ["innovative",        ["innovational"]],
        ["inquisitorial",     ["inquisitional"]],
        ["insistence",        ["insistment"]],
        ["instillation",      ["instillment"]],
        ["instinctive",       ["instinctual"]],
        ["insubstantial",     ["unsubstantial"]],
        ["insurer",           ["insuror"]],
        ["insurrectionary",   ["insurrectional"]],
        ["interpret",         ["interpretate"]],
        ["intervention",      ["intervenience"]],
        ["ironic",            ["ironical"]],
        ["irrevocable",       ["unrevokable"]],
        ["judgmental",        ["judgmatic"]],
        ["jury-rigged",       ["gerry-rigged"]],
        ["jury-rigged",       ["jerry-rigged"]],
        ["kaffeeklatsch",     ["Coffee klatsch", "coffee klatch"]],
        ["knickknack",        ["nicknack"]],
        ["labyrinthine",      ["labyrinthian"]],
        ["laudatory",         ["laudative"]],
        ["legitimation",      ["legitimatization"]],
        ["legitimation",      ["legitimization"]],
        ["legitimize",        ["legitimatize"]],
        ["lengthwise",        ["lengthways"]],
        ["licorice",          ["liquorice"]],
        ["life-size",         ["life-sized"]],
        ["lithe",             ["lithesome"]],
        ["loath",             ["loth"]],
        ["lollypop",          ["lollipop"]],
        ["lubricious",        ["lubricous"]],
        ["mayhem",            ["maihem"]],
        ["medical marijuana", ["medicinal marijuana"]],
        ["minimize",          ["minimalize"]],
        ["monetize",          ["monetarize"]],
        ["movable",           ["moveable"]],
        ["murk",              ["mirk"]],
        ["murky",             ["mirky"]],
        ["narcissism",        ["narcism"]],
        ["neglectful",        ["neglective"]],
        ["negligence",        ["negligency"]],
        ["neologist",         ["neologizer"]],
        ["neurological",      ["neurologic"]],
        ["nictitate",         ["nictate"]],
        ["normality",         ["normalcy"]],
        ["numbness",          ["numbedness"]],
        ["omissible",         ["omittable"]],
        ["onomatopoeic",      ["onomatopoetic"]],
        ["opined",            ["opinioned"]],
        ["optimal advantage", ["optimum advantage"]],
        ["orient",            ["orientate"]],
        ["outsize",           ["outsized"]],
        ["oversize",          ["oversized"]],
        ["overthrow",         ["overthrowal"]],
        ["pacifist",          ["pacificist"]],
        ["parti-colored",     ["parti-color"]],
        ["parti-colored",     ["party-colored"]],
        ["participatory",     ["participative"]],
        ["partner",           ["copartner"]],
        ["partnership",       ["copartnership"]],
        # ["password",          ["passcode"]],  # FIXME
        ["patina",            ["patine"]],
        ["pederast",          ["paederast"]],
        ["pediatrician",      ["pediatrist"]],
        ["pejorative",        ["perjorative"]],
        ["penumbral",         ["penumbrous"]],
        ["permissive",        ["permissory"]],
        ["permute",           ["permutate"]],
        ["pharmaceutical",    ["pharmaceutic"]],
        ["pleurisy",          ["pleuritis"]],
        ["policyholder",      ["policy holder"]],
        ["policyholder",      ["policyowner"]],
        ["politicize",        ["politicalize"]],
        ["pre-Columbian",     ["precolumbian"]],
        ["precedence",        ["precedency"]],
        ["preceptorial",      ["preceptoral"]],
        ["precipitancy",      ["precipitance"]],
        ["precipitate",       ["precipitant"]],
        ["preclusive",        ["preclusory"]],
        ["prefectorial",      ["prefectoral"]],
        ["preponderantly",    ["preponderately"]],
        ["preservation",      ["preserval"]],
        ["preventive",        ["preventative"]],
        ["proconsulate",      ["proconsulship"]],
        ["procreative",       ["procreational"]],
        ["procurement",       ["procurance"]],
        ["propulsion",        ["propelment"]],
        ["propulsive",        ["propulsory"]],
        ["prosecutory",       ["prosecutive"]],
        ["protective",        ["protectory"]],
        ["provocative",       ["provocatory"]],
        ["prurience",         ["pruriency"]],
        ["psychical",         ["psychal"]],
        ["punitive",          ["punitory"]],
        ["pygmy",             ["pygmean", "pygmaen"]],
        ["quantify",          ["quantitate"]],
        ["questionnaire",     ["questionary"]],
        ["quiescence",        ["quiescency"]],
        ["rabbi",             ["rabbin"]],
        ["reasonableness",    ["reasonability"]],
        ["recidivous",        ["recidivistic"]],
        ["recriminatory",     ["recriminative"]],
        ["recruitment",       ["recruital"]],
        ["recurrence",        ["recurrency"]],
        ["recusal",           ["recusation"]],
        ["recusal",           ["recusement"]],
        ["recusancy",         ["recusance"]],
        ["redemptive",        ["redemptory"]],
        ["referable",         ["referrable"]],
        ["referable",         ["referrible"]],
        ["refutative",        ["refutatory"]],
        ["remission",         ["remittal"]],
        ["remittance",        ["remitment"]],
        ["renounceable",      ["renunciable"]],
        ["renunciation",      ["renouncement"]],
        ["reparative",        ["reparatory"]],
        ["repudiatory",       ["repudiative"]],
        ["requital",          ["requitement"]],
        ["rescission",        ["rescindment"]],
        ["restoration",       ["restoral"]],
        ["reticence",         ["reticency"]],
        ["retributive",       ["retributional", "retributionary"]],
        ["review",            ["reviewal"]],
        ["revision",          ["revisal"]],
        ["revisionary",       ["revisional"]],
        ["revocable",         ["revokable", "revokeable"]],
        ["revolt",            ["revolute"]],
        ["salience",          ["saliency"]],
        ["salutary",          ["salutiferous"]],
        ["sensory",           ["sensatory"]],
        ["sessional",         ["sessionary"]],
        ["shareholder",       ["shareowner"]],
        ["sickly",            ["sicklily"]],
        ["signatory",         ["signator"]],
        ["slander",           ["slanderize"]],
        ["societal",          ["societary"]],
        ["sodomite",          ["sodomist"]],
        ["solicit",           ["solicitate"]],
        ["speculative",       ["speculatory"]],
        ["spirituous",        ["spiritous"]],
        ["statutory",         ["statutorial"]],
        ["submersible",       ["submergeable"]],
        ["submission",        ["submittal"]],
        ["subtle",            ["subtile"]],
        ["succubus",          ["succuba"]],
        ["sufficiency",       ["sufficience"]],
        ["supplicant",        ["suppliant"]],
        ["surmise",           ["surmisal"]],
        ["suspendable",       ["suspendible"]],
        ["swathe",            ["enswathe"]],
        ["synthesize",        ["synthetize"]],
        ["systematize",       ["systemize"]],
        ["T-shirt",           ["tee-shirt"]],
        ["tactile",           ["tactual"]],
        ["tangential",        ["tangental"]],
        ["tautological",      ["tautologous"]],
        ["thenceforth",       ["thenceforward"]],
        ["transience",        ["transiency"]],
        ["transposition",     ["transposal"]],
        ["transposition",     ["transposal"]],
        ["unalterable",       ["inalterable"]],
        ["uncommunicative",   ["incommunicative"]],
        ["uncontrollable",    ["incontrollable"]],
        ["unenforceable",     ["nonenforceable"]],
        ["unnavigable",       ["innavigable"]],
        ["unreasonableness",  ["unreasonability"]],
        ["unsolvable",        ["insolvable"]],
        ["usurpation",        ["usurpature"]],
        ["variational",       ["variative"]],
        ["vegetative",        ["vegetive"]],
        ["vindictive",        ["vindicative"]],
        ["vituperative",      ["vituperous"]],
        ["vociferous",        ["vociferant"]],
        ["volitional",        ["volitive"]],
        ["wolfish",           ["wolvish"]],
        ["wolverine",         ["wolverene"]],
        ["Zoroastrianism",    ["Zoroastrism"]],
    ]

    return preferred_forms_check(text, preferences, err, msg)



# -*- coding: utf-8 -*-
"""Misspellings.

---
layout:     post
source:     Garner's Modern American Usage
source_url: http://bit.ly/1T4alrY
title:      misspellings
date:       2014-06-10 12:31:19
categories: writing
---

Points out misspellings.

"""
from proselint.tools import memoize, preferred_forms_check


@memoize
def check(text):
    """Suggest the preferred forms."""
    err = "spelling.misc"
    msg = "Misspelling. '{}' is the preferred spelling."

    misspellings = [
        ["a lot",             ["alot"]],
        ["academically",      ["academicly"]],
        ["accidentally",      ["accidently"]],
        ["accommodable",      ["accomodatable"]],
        ["anilingus",         ["analingus"]],
        ["aren't i",          ["amn't i"]],
        ["aren't i",          ["an't i"]],
        ["barbed wire",       ["bob wire"]],
        ["chargeable",        ["chargable"]],
        ["chiffonier",        ["chiffonnier"]],
        ["chintzy",           ["chinchy"]],
        ["chipotle",          ["chipolte"]],
        ["chlorophyll",       ["chlorophyl"]],
        ["chocolaty",         ["chocolatey"]],
        ["chronicle",         ["chronical"]],
        ["chronicles",        ["chronicals"]],
        ["coleslaw",          ["coldslaw"]],
        ["choosy",            ["choosey"]],
        ["cummerbund",        ["kummerbund"]],
        ["financeable",       ["financable"]],
        ["fluorescent",       ["flourescent"]],
        ["fluoridation",      ["flouridation"]],
        ["fluoride",          ["flouride"]],
        ["foreclose",         ["forclose"]],
        ["forswear",          ["foreswear"]],
        ["free rein",         ["free reign"]],
        ["gist",              ["jist"]],
        ["glamour",           ["glamor"]],
        ["granddad",          ["grandad"]],
        ["grandpa",           ["granpa"]],
        ["highfalutin",       ["highfaluting", "highfalutin'", "hifalutin"]],
        ["Hippocratic",       ["hypocratic"]],
        ["hirable",           ["hireable"]],
        ["holistic",          ["wholistic"]],
        ["ideology",          ["idealogy"]],
        ["idiosyncrasy",      ["ideosyncracy"]],
        ["improvise",         ["improvize"]],
        ["incurrence",        ["incurment"]],
        ["inevitability",     ["inevitableness"]],
        ["innovate",          ["inovate"]],
        ["inoculation",       ["innoculation", "inocculation"]],
        ["integral",          ["intergral"]],
        ["inundate",          ["innundate"]],
        ["inundated",         ["innundated"]],
        ["inundates",         ["innundates"]],
        ["inundating",        ["innundating"]],
        ["iridescent",        ["irridescent"]],
        ["irrelevant",        ["irrevelant"]],
        ["jujitsu",           ["jiujutsu"]],
        ["kaleidoscope",      ["kaleidascope"]],
        ["knickknack",        ["knicknack"]],
        ["lassos",            ["lassoes"]],
        ["lessee",            ["leasee"]],
        ["lessor",            ["leasor"]],
        ["liaison",           ["liason"]],
        ["liaison",           ["laison"]],
        ["lightning rod",     ["lightening rod"]],
        ["struck by lightning", ["struck by lightening"]],
        ["liquefy",           ["liquify"]],
        ["loathsome",         ["loathesome"]],
        ["lodestar",          ["loadstar"]],
        ["to make do",        ["to make due"]],
        ["mademoiselle",      ["madamoiselle"]],
        ["martial arts",      ["marshall arts"]],
        ["court-martial",     ["court marshall", "court-marshall"]],
        ["maelstrom",         ["maelstorm"]],
        ["mafia",             ["maffia"]],
        ["mafiosi",           ["mafiosos"]],
        ["mangoes",           ["mangos"]],
        ["marijuana",         ["marihuana"]],
        ["marshmallow",       ["marshmellow"]],
        ["martial law",       ["marshall law"]],
        ["massacring",        ["massacering", "massacreing"]],
        ["measles",           ["measels"]],
        ["memento",           ["momento"]],
        ["minuscule",         ["miniscule"]],
        ["mileage",           ["milage"]],
        ["milieu",            ["mileau"]],
        ["millennium",        ["millenium"]],
        ["millennia",         ["millenia"]],
        ["millenniums",       ["milleniums"]],
        ["millipede",         ["millepede"]],
        ["millionaire",       ["millionnaire"]],
        ["milquetoast",       ["milktoast"]],
        ["mimicked",          ["mimiced"]],
        ["mimicking",         ["mimicing"]],
        ["misspelling",       ["mispelling"]],
        ["mischievous",       ["mischievious"]],
        ["moccasin",          ["mocasin", "moccassin", "mocassin"]],
        ["monologue",         ["monolog"]],
        ["monologuist",       ["monologist"]],
        ["naphtha",           ["naptha"]],
        ["navel orange",      ["naval orange"]],
        ["neckwear",          ["neckware"]],
        ["newsstand",         ["newstand"]],
        ["non sequitur",      ["non sequiter",
                               "non sequitar",
                               "non sequitor"]],
        ["nouveau riche",     ["nouveau rich"]],
        ["occurred",          ["occured"]],
        ["plantar fasciitis", ["planter fasciitis", "plantar fascitis"]],
        ["pledgeable",        ["pledgable", "pledgeble"]],
        ["portentous",        ["portentuous", "portentious"]],
        ["praying mantis",    ["preying mantis"]],
        ["preestablished",    ["prestablished"]],
        ["preexisting",       ["prexisting"]],
        ["presumptuous",      ["presumptious"]],
        ["rarefy",            ["rarify"]],
        ["reckless",          ["wreckless"]],
        ["remuneration",      ["renumeration"]],
        ["restaurateur",      ["restauranteur"]],
        ["retractable",       ["retractible"]],
        ["reverie",           ["revery"]],
        ["spicy",             ["spicey"]],
        ["stupefy",           ["stupify"]],
        ["subtly",            ["subtley"]],
        ["till",              ["\'till?"]],
        ["tinderbox",         ["tenderbox"]],
        ["timpani",           ["tympani"]],
        ["a timpani",         ["a timpano"]],
        ["umpteenth",         ["umteenth"]],
        ["verbiage",          ["verbage"]],
    ]

    return preferred_forms_check(text, misspellings, err, msg)
